/*
Copyright © 2018 Roesner P/L

i4M Machine Controller
*/

var i2c = require('i2c-bus');
var fs = require('fs');

//log to console if it exists, else set console.log to nop
if (console){
	console.log('Console log enabled');
}
else {
	console.log = function(){};
}

//i2c comms to main board
var i2c1 = i2c.openSync(1);

//buffer for command data to main board
var txData1 = new Buffer(32, 0);

var hardwareVer;
var softwareVer;
var cycleDataTimer;
var cycleCounter = 0;
var saveConfigTimer;

var loadCellMcuOnline = true;
var mainMcuOnline = true;
var i2cLoadCellCounts = 0;
var i2cMainMcuCounts = 0;
var ledMode = 0;

var i2cLoadCellFlags = 0;
var i2cLoadCell0 = 0;
var i2cLoadCell1 = 0;
var i2cLoadCell2 = 0;
var i2cLoadCell3 = 0;
var resolvedLoadCell = 0;
var displayWeight = 0;

var i2cMainMcuFlags = 0;
var i2cSupplyVoltage = 0;
var i2cDriveWheelSpeed = 0;
var i2cSpinnerSpeed = 0;
var i2cDoorSensorMv = 0;

var actualRate = 0;
var rawRate = 0;
var spinnerRpm = 0;
var targetDoorHeightMm = 0;
var actualDoorHeightMm = 0;
var rollerRevsPerHa = 0;
var pulsesPerRollerRev = 0;
var instantHectares = 0;
var spreading = false;

var alert1 = false;
var alert2 = false;
var alert3 = false;
var alert4 = false;
var alert5 = false;
var alert6 = false;
var alert7 = false;
var alert1Counter = 0;
var alert2Counter = 0;
var alert3Counter = 0;
var alert4Counter = 0;
var alert5Counter = 0;
var alert6Counter = 0;
var alert7Counter = 0;

//tracking filter
var deadBand = 0;
var featherTime = 0;
var fineAdjustStart = 0;
var fineAdjustUp = 0;
var fineAdjustDown = 0;
var coarseAdjust = 0;
var tooLowCounter = 0;
var tooHighCounter = 0;

//moving average filter
var sampleBuffer = new Array(100).fill(0);
var numberOfSamples = 100;
var samples = 0;
var samplePointer = 0;
var sampleTotal = 0;
var resetFilter = true;

//data file variables
var countdownKg = 0;
var calSampleKg = 0;
var hectareCounter = 0;

//config file variables
var loadCellsEnabled;
var quickFillKgs;
var useLoadCell0;
var useLoadCell1;
var useLoadCell2;
var useLoadCell3;
var loadCellZero;
var loadCellSpan;
var weightSpan;
var weightOffset;
var filterMode;
var doorWidth;
var rollerDiameter;
var driveWheelPulsesPerRev;
var distancePerPulse;
var actuatorInstalled;
var gear1Ratio;
var gear2Ratio;
var gear3Ratio;
var gear4Ratio;
var storedProducts;
var storedRates;
var storedWidths;
var storedGearSettings;
var storedCals;
var productPointer;
var productName;
var targetRate;
var spreadWidth;
var gearSetting;
var productCal;

readVersionFile();
readDataStorageFile();
loadUserConfig();
//loadcell config not required for now, adc reads all 4 channels
//setTimeout(configLoadCellMcu, 1000);
//set controller led to solid green
ledMode = 2;
setTimeout(setLedMode, 1200);
setTimeout(configMainMcu, 1500);
setTimeout(cycleData, 2000);

function readVersionFile(){
	try {
		//read and parse the version file
		var rawData = fs.readFileSync('/home/pi/servermain/version.json');
		var version = JSON.parse(rawData);
		
		hardwareVer = version.hardwareVer;
		softwareVer = version.softwareVer;
	}
	catch (e){
		console.log('Catch error readVersionFile');
	}
}

function readDataStorageFile(){
	try {
		//read and parse the dataStorage file
		var rawData = fs.readFileSync('/home/pi/servermain/appFR/dataStorage.json');
		var dataStorage = JSON.parse(rawData);
		
		countdownKg = dataStorage.countdownKg;
		calSampleKg = dataStorage.calSampleKg;
		hectareCounter = dataStorage.hectareCounter;
	}
	catch (e){
		console.log('Catch error readDataStorageFile');
	}
}

function saveDataStorageFile(){
	try {
		//load variables to save to dataStorage
		var dataStorage = {
			countdownKg: countdownKg,
			calSampleKg: calSampleKg,
			hectareCounter: hectareCounter
		};
		
		//convert to json strings and write to file
		var data = JSON.stringify(dataStorage, null, 2);
		fs.writeFileSync('/home/pi/servermain/appFR/dataStorage.json', data);
		
		console.log('Saving dataStorage file');
	}
	catch (e){
		console.log('Catch error saveDataStorageFile');
	}
}

function clearDataStorage(){
	try {
		//clear stored data
		countdownKg = 0;
		calSampleKg = 0;
		hectareCounter = 0;
		
		saveDataStorageFile();
	}
	catch (e){
		console.log('Catch error clearDataStorage');
	}
}

function loadUserConfig(){
	try {
		//read and parse the config file
		var rawData = fs.readFileSync('/home/pi/servermain/appFR/configUser.json');
		var config = JSON.parse(rawData);
		
		loadCellsEnabled = config.loadCellsEnabled;
		quickFillKgs = config.quickFillKgs;
		useLoadCell0 = config.useLoadCell0;
		useLoadCell1 = config.useLoadCell1;
		useLoadCell2 = config.useLoadCell2;
		useLoadCell3 = config.useLoadCell3;
		loadCellZero = config.loadCellZero;
		loadCellSpan = config.loadCellSpan;
		weightSpan = config.weightSpan;
		weightOffset = config.weightOffset;
		filterMode = config.filterMode;
		doorWidth = config.doorWidth;
		rollerDiameter = config.rollerDiameter;
		driveWheelPulsesPerRev = config.driveWheelPulsesPerRev;
		distancePerPulse = config.distancePerPulse;
		actuatorInstalled = config.actuatorInstalled;
		gear1Ratio = config.gear1Ratio;
		gear2Ratio = config.gear2Ratio;
		gear3Ratio = config.gear3Ratio;
		gear4Ratio = config.gear4Ratio;
		storedProducts = config.storedProducts;
		storedRates = config.storedRates;
		storedWidths = config.storedWidths;
		storedGearSettings = config.storedGearSettings;
		storedCals = config.storedCals;
		productPointer = config.productPointer;
		
		//set product variables based on the stored pointer
		productName = storedProducts[productPointer];
		targetRate = storedRates[productPointer],
		spreadWidth = storedWidths[productPointer];
		gearSetting = storedGearSettings[productPointer];
		productCal = storedCals[productPointer];

		console.log('Loading user config');
		console.log(config);
	}
	catch (e){
		//config file is blank or faulty, so load the factory data
		loadFactoryConfig();
		console.log('Catch error loadUserConfig');
	}
}

function loadFactoryConfig(){
	try {
		//read and parse the config file
		var rawData = fs.readFileSync('/home/pi/servermain/appFR/configFactory.json');
		var config = JSON.parse(rawData);
		
		loadCellsEnabled = config.loadCellsEnabled;
		quickFillKgs = config.quickFillKgs;
		useLoadCell0 = config.useLoadCell0;
		useLoadCell1 = config.useLoadCell1;
		useLoadCell2 = config.useLoadCell2;
		useLoadCell3 = config.useLoadCell3;
		loadCellZero = config.loadCellZero;
		loadCellSpan = config.loadCellSpan;
		weightSpan = config.weightSpan;
		weightOffset = config.weightOffset;
		filterMode = config.filterMode;
		doorWidth = config.doorWidth;
		rollerDiameter = config.rollerDiameter;
		driveWheelPulsesPerRev = config.driveWheelPulsesPerRev;
		distancePerPulse = config.distancePerPulse;
		actuatorInstalled = config.actuatorInstalled;
		gear1Ratio = config.gear1Ratio;
		gear2Ratio = config.gear2Ratio;
		gear3Ratio = config.gear3Ratio;
		gear4Ratio = config.gear4Ratio;
		storedProducts = config.storedProducts;
		storedRates = config.storedRates;
		storedWidths = config.storedWidths;
		storedGearSettings = config.storedGearSettings;
		storedCals = config.storedCals;
		productPointer = config.productPointer;
		
		//set product variables based on the stored pointer
		productName = storedProducts[productPointer];
		targetRate = storedRates[productPointer],
		spreadWidth = storedWidths[productPointer];
		gearSetting = storedGearSettings[productPointer];
		productCal = storedCals[productPointer];

		console.log('Loading factory config');
		console.log(config);
	}
	catch (e){
		console.log('Catch error loadFactoryConfig');
	}
}

function factoryLoadCells(){
	try {
		//read and parse the config file
		var rawData = fs.readFileSync('/home/pi/servermain/appMON/configFactory.json');
		var config = JSON.parse(rawData);
		
		useLoadCell0 = config.useLoadCell0;
		useLoadCell1 = config.useLoadCell1;
		useLoadCell2 = config.useLoadCell2;
		useLoadCell3 = config.useLoadCell3;
		loadCellZero = config.loadCellZero;
		loadCellSpan = config.loadCellSpan;
		weightSpan = config.weightSpan;
		weightOffset = config.weightOffset;
		filterMode = config.filterMode;

		console.log('Loading factory loadcells');
		console.log(config);
	}
	catch (e){
		console.log('Catch error loadFactoryLoadCell');
	}
}

function saveUserConfig(){
	try {
		//load variables used by pi and main board
		var configData = {
			loadCellsEnabled: loadCellsEnabled,
			quickFillKgs: quickFillKgs,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode,
			doorWidth: doorWidth,
			rollerDiameter: rollerDiameter,
			driveWheelPulsesPerRev: driveWheelPulsesPerRev,
			distancePerPulse: distancePerPulse,
			actuatorInstalled: actuatorInstalled,
			gear1Ratio: gear1Ratio,
			gear2Ratio: gear2Ratio,
			gear3Ratio: gear3Ratio,
			gear4Ratio: gear4Ratio,
			storedProducts: storedProducts,
			storedRates: storedRates,
			storedWidths: storedWidths,
			storedGearSettings: storedGearSettings,
			storedCals: storedCals,
			productPointer: productPointer
		};
		
		//convert to json strings and write to file
		var data = JSON.stringify(configData, null, 2);
		fs.writeFileSync('/home/pi/servermain/appFR/configUser.json', data);
		
		console.log('Saving user config file');
	}
	catch (e){
		console.log('Catch error saveUserConfig');
	}
}

function saveFactoryConfig(){
	try {
		//load variables used by pi and main board
		var configData = {
			loadCellsEnabled: loadCellsEnabled,
			quickFillKgs: quickFillKgs,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode,
			doorWidth: doorWidth,
			rollerDiameter: rollerDiameter,
			driveWheelPulsesPerRev: driveWheelPulsesPerRev,
			distancePerPulse: distancePerPulse,
			actuatorInstalled: actuatorInstalled,
			gear1Ratio: gear1Ratio,
			gear2Ratio: gear2Ratio,
			gear3Ratio: gear3Ratio,
			gear4Ratio: gear4Ratio,
			storedProducts: storedProducts,
			storedRates: storedRates,
			storedWidths: storedWidths,
			storedGearSettings: storedGearSettings,
			storedCals: storedCals,
			productPointer: productPointer
		};
		
		//convert to json strings and write to file
		var data = JSON.stringify(configData, null, 2);
		fs.writeFileSync('/home/pi/servermain/appFR/configFactory.json', data);
		
		console.log('Saving factory config file');
	}
	catch (e){
		console.log('Catch error saveFactoryConfig');
	}
}

function setLedMode(){
	try {
		//led modes 1 = solid red, 2 = solid green, 3 = flashing red, 4 = flashing green
		//command bytes to send via i2c
		txData1[0] = 0xc1;
		txData1[1] = 0x00;
		txData1[2] = 0x03;
		txData1[3] = ledMode;
		txData1[4] = 0x00;
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	}
	catch (e){
		console.log('Catch error setLedMode');
	}
}

function configLoadCellMcu(){
	try {
		var configFlags = 0;
		if (useLoadCell0 == true){
			configFlags = configFlags | 0x01;
		}
		if (useLoadCell1 == true){
			configFlags = configFlags | 0x02;
		}
		if (useLoadCell2 == true){
			configFlags = configFlags | 0x04;
		}
		if (useLoadCell3 == true){
			configFlags = configFlags | 0x08;
		}
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x02;
		txData1[3] = 0x00;
		txData1[4] = lowByte(configFlags);
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to loadcell mcu
		writeLoadCellMcu(txData1);
	}
	catch (e){
		console.log('Catch error configLoadCellMcu');
	}
}


function configMainMcu(){
	try {
		//not required
	}
	catch (e){
		console.log('Catch error configMainMcu');
	}
}

//called regularly to get data from the main board
function cycleData(){
	try {
		//if the slave is online, read it
		if (loadCellMcuOnline){
			readLoadCellMcu();
		}
		
		//process loadcells if they are enabled
		//allow time for mainboard to respond min 5ms
		if (loadCellsEnabled){
			setTimeout(processLoadCells, 30);
		}
		
		//we read loadcells more often than the main mcu (for the filter)
		//so after some number of loadcell reads, read the main mcu
		cycleCounter ++;
		
		if (cycleCounter == 5){
			cycleCounter = 0;
			if (mainMcuOnline){
				readMainMcu();
				//allow time for mainboard to respond min 5ms
				setTimeout(processMainMcu, 30);
			}
		}
		
		//run again after this delay
		cycleDataTimer = setTimeout(cycleData, 200);
	}
	catch (e){
		console.log('Catch error cycleData');
	}
}

function readLoadCellMcu(){
	try {
		//message settings
		var slaveID = 80;
		var slaveRegister = 100;
		var packetLength = 15;
		var packet = new Buffer(32, 0);
		
		//read block of bytes
		i2c1.readI2cBlockSync(slaveID, slaveRegister, packetLength, packet);
		
		//if valid end of frame byte
		//clr failure counter, and transfer the data
		if (packet[14] == 0xC3){
			loadCellMcuOnline = true;
			i2cLoadCellCounts ++;
			
			console.log('I2C mcu2 read loadcells:   OK   ' + i2cLoadCellCounts);
			
			i2cLoadCellFlags = packet[1];
			i2cLoadCell0 = join3Bytes(packet[2], packet[3], packet[4]);
			i2cLoadCell1 = join3Bytes(packet[5], packet[6], packet[7]);
			i2cLoadCell2 = join3Bytes(packet[8], packet[9], packet[10]);
			i2cLoadCell3 = join3Bytes(packet[11], packet[12], packet[13]);
		}
		else {
			console.log('End of frame error readLoadCellMcu');
			//reset this flag after 2secs, this resets the mcu i2c module
			loadCellMcuOnline = false;
			setTimeout(function(){ loadCellMcuOnline = true; }, 2000);
		}
	}
	catch (e){
		console.log('Catch error readLoadCellMcu');
		//reset this flag after 2secs, this resets the mcu i2c module
		loadCellMcuOnline = false;
		setTimeout(function(){ loadCellMcuOnline = true; }, 2000);
	}
}

function processLoadCells(){
	try	{
		//sum the active loadcell channels and div by number of channels
		var average = 0;
		var channels = 0;
		var tempDisplayWeight = displayWeight;
		
		if (useLoadCell0 == true){
			average += i2cLoadCell0;
			channels ++;
		}
		if (useLoadCell1 == true){
			average += i2cLoadCell1;
			channels ++;
		}
		if (useLoadCell2 == true){
			average += i2cLoadCell2;
			channels ++;
		}
		if (useLoadCell3 == true){
			average += i2cLoadCell3;
			channels ++;
		}
		
		average = Math.round(average / channels);
		resolvedLoadCell = average;
		
		//if loadcell range is not zero, calc the weight
		if (loadCellSpan - loadCellZero){
			
			var newWeight = weightSpan * (resolvedLoadCell - loadCellZero);
			newWeight = newWeight / (loadCellSpan - loadCellZero);
			newWeight = parseInt(newWeight, 10);
			
			//apply the weight offset now
			newWeight += weightOffset;
			
			if (resetFilter){
				//reset data for moving ave filter
				sampleBuffer.fill(0);
				samples = 0;
				samplePointer = 0;
				sampleTotal = 0;
				resetFilter = false;
			}
			
			switch (filterMode){
				
				//filter off
				case 0:
					tempDisplayWeight = newWeight;
				break;
				
				
				//tracking filter, if display is too low, it will gradually increase
				//until feather time elapses, then apply coarse adjust
				case 1:
					featherTime = 15;
					deadBand = 5;
					fineAdjustStart = 4;
					fineAdjustUp += 2;
					fineAdjustDown += 2;
					coarseAdjust = 200;
					
					//too high
					if (displayWeight > (newWeight + deadBand))
					{
						tooHighCounter ++;
						tooLowCounter = 0;
						fineAdjustUp = 0;
						
						if (tooHighCounter < featherTime)
						{	
							tempDisplayWeight -= fineAdjustStart + fineAdjustDown;
						}
						else
						{
							tempDisplayWeight -= coarseAdjust;
							fineAdjustDown = 0;
							
							if (tempDisplayWeight < newWeight)
							{
								tempDisplayWeight = newWeight;
								tooHighCounter = 0;
							}
						}
					}
					//too low
					else if (displayWeight < (newWeight - deadBand))
					{
						tooLowCounter ++;
						tooHighCounter = 0;
						fineAdjustDown = 0;
						
						if (tooLowCounter < featherTime)
						{
							tempDisplayWeight += fineAdjustStart + fineAdjustUp;
						}
						else
						{
							tempDisplayWeight += coarseAdjust;
							fineAdjustUp = 0;
							
							if (tempDisplayWeight > newWeight)
							{
								tempDisplayWeight = newWeight;
								tooLowCounter = 0;
							}
						}
					}
					else
					{
						tooHighCounter = 0;
						tooLowCounter = 0;
						fineAdjustUp = 0;
						fineAdjustDown = 0;
					}
				break;
				
				
				//moving average filter
				case 2:
					numberOfSamples = 50;
					
					if (samples < numberOfSamples)
					{
						samples ++;
					}
					
					sampleTotal -= sampleBuffer[samplePointer];
					sampleTotal += newWeight;
					sampleBuffer[samplePointer] = newWeight;
					
					tempDisplayWeight = sampleTotal / samples;
					
					samplePointer ++;
					
					if (samplePointer == numberOfSamples)
					{
						samplePointer = 0;
					}
				break;
				
				default:
			}
			
			//if filter is on, snap to zero if within 50kg
			if (filterMode){
				if (tempDisplayWeight < 50 && tempDisplayWeight > -50){
					tempDisplayWeight = 0;
				}
			}
		}
		else {
			tempDisplayWeight = 0;
		}
		
		//update display, round to 1kg
		//displayWeight = Math.round(tempDisplayWeight);
		
		//if zero or span is the default value, load a big negative number
		//so the display prompts user to calibrate
		if (loadCellZero == 0 || loadCellSpan == 130000){
			displayWeight = -100000;
		}
		else {
			//update display, round to 10kg
			displayWeight = Math.round(tempDisplayWeight / 10) * 10;
		}
		
	}
	catch (e){
		console.log('Catch error processLoadCells');
	}
}

function writeLoadCellMcu(txData){
	try	{
		//message settings
		var slaveID = 80;
		var slaveRegister = 0;
		var packetLength = 15;
		
		//load end of frame byte
		txData[14] = 0xC3;
		
		//send block of bytes
		i2c1.writeI2cBlockSync(slaveID, slaveRegister, packetLength, txData);
		
		console.log('I2C loadcell mcu:          WRITE');
		
		//i2c1.closeSync();
	}
	catch (e){
		console.log('Catch error writeLoadCellMcu');
	}
}

function writeMainMcu(txData){
	try	{
		//message settings
		var slaveID = 60;
		var slaveRegister = 0;
		var packetLength = 15;
		
		//load end of frame byte
		txData[14] = 0xC3;
		
		//send block of bytes
		i2c1.writeI2cBlockSync(slaveID, slaveRegister, packetLength, txData);
		
		console.log('I2C main mcu:              WRITE');
		
		//i2c1.closeSync();
	}
	catch (e){
		console.log('Catch error writeMainMcu');
	}
}

function readMainMcu(){
	try	{
		//message settings
		var slaveID = 60;
		var slaveRegister = 100;
		var packetLength = 30;
		var packet = new Buffer(32, 0);
		
		//read block of bytes
		i2c1.readI2cBlockSync(slaveID, slaveRegister, packetLength, packet);
		
		//if valid end of frame byte
		//clr failure counter, and transfer the data
		if (packet[29] == 0xC3){
			mainMcuOnline = true;
			i2cMainMcuCounts ++;
				
			console.log('I2C mcu1 read data1:       OK   ' + i2cMainMcuCounts);
			
			i2cMainMcuFlags = join2bytes(packet[1], packet[2]);
			i2cSupplyVoltage = packet[3];
			i2cDriveWheelSpeed = join2bytes(packet[4], packet[5]);
			i2cSpinnerSpeed = join2bytes(packet[6], packet[7]);
			i2cDoorSensorMv = join2bytes(packet[10], packet[11]);
		}
		else {
			console.log('End of frame error readMainMcu');
			//reset this flag after 2secs, this resets the mcu i2c module
			mainMcuOnline = false;
			setTimeout(function(){ mainMcuOnline = true; }, 2000);
		}
	}
	catch (e){
		console.log('Catch error readMainMcu');
		//reset this flag after 2secs, this resets the mcu i2c module
		mainMcuOnline = false;
		setTimeout(function(){ mainMcuOnline = true; }, 2000);
	}
}


//basic operation
//user selects product, rate, width, gear then taps 'set door'
//when drivewheel is turning the actual rate is displayed, hectares accumulate
//cal sample accumulates, alerts are active


function processMainMcu(){
	try {
		/*
		//test values 20mm door, 10hz wheel, 720rpm spinner
		i2cDriveWheelSpeed = 1000;
		i2cDoorSensorMv = 9574;
		i2cSpinnerSpeed = 1200;
		*/
		
		//update if we have started or stopped spreading
		startStop();
		
		//calc actual door height
		if (actuatorInstalled){
			actualDoorHeightMm = 245 - (i2cDoorSensorMv / 42.553);
		}
		else {
			actualDoorHeightMm = targetDoorHeightMm;
		}
		
		//refresh target door height
		calcDoor();
		
		//calc actual rate to display
		calcActualRate();
		
		//spinner speed rpm rounded to nearest 10
		spinnerRpm = Math.round(i2cSpinnerSpeed * 0.06) * 10;
		
		//check if we need to fire any alerts
		processAlerts();
	}
	catch (e){
		console.log('Catch error processMainMcu');
	}
}


function startStop(){
	try {
		//if the belt has stopped, turn spreading flag off, save hectares etc
		if (spreading){
			if (!i2cDriveWheelSpeed){
				spreading = false;
				saveDataStorageFile();
			}
		}
		//else if the belt is turning, turn spreading flag on
		else {
			if (i2cDriveWheelSpeed){
				spreading = true;
			}
		}
	}
	catch (e){
		console.log('Catch error startStop');
	}
}


function calcActualRate(){
	try	{
		//first calc kg dispensed for the cal sample, then calc the actual rate
		var actualDoorHeightM = actualDoorHeightMm / 1000;
		
		//calc the drive wheel pulses for 1 roller revolution
		if (gearSetting == 1){
			pulsesPerRollerRev = driveWheelPulsesPerRev * gear1Ratio;
		}
		else if (gearSetting == 2){
			pulsesPerRollerRev = driveWheelPulsesPerRev * gear2Ratio;
		}
		else if (gearSetting == 3){
			pulsesPerRollerRev = driveWheelPulsesPerRev * gear3Ratio;
		}
		else if (gearSetting == 4){
			pulsesPerRollerRev = driveWheelPulsesPerRev * gear4Ratio;
		}
		
		//calc the kg dispensed per second, div 100 because i2cDriveWheelSpeed is x100
		var kgDispensed = (i2cDriveWheelSpeed * doorWidth * actualDoorHeightM * rollerDiameter * 3.14159265359 * productCal) / (pulsesPerRollerRev * 100);
		
		//update the cal sample weight
		calSampleKg += kgDispensed;
		
		//if not using loadcells, process the kg counter
		if (!loadCellsEnabled){
			
			countdownKg -= kgDispensed;
			
			if (countdownKg < 0){
				countdownKg = 0;
				displayWeight = 0;
			}
			else {
				//update display, round to 10kg
				displayWeight = Math.round(countdownKg / 10) * 10;
			}
		}
		
		//if zero target rate, or zero belt speed, show zero
		if (!targetRate || !i2cDriveWheelSpeed){
			actualRate = 0;
			rawRate = 0;
			return;
		}
		
		//calc the ha covered, i2cDriveWheelSpeed is pps x100
		if (i2cDriveWheelSpeed > 0){
			instantHectares = (i2cDriveWheelSpeed * distancePerPulse * spreadWidth) / 1000000;
		}
		else {
			instantHectares = 0;
		}
		
		//calc the rate
		if (instantHectares > 0){
			rawRate = Math.round(kgDispensed / instantHectares);
		}
		else {
			rawRate = 0;
		}
		
		//accumulate hectare counter while we're here
		hectareCounter += instantHectares;
		
		
		//light method to smooth and snap to target rate
		if (rawRate && targetRate){
			
			var rawRatio = rawRate / targetRate;
			
			//if equal or above the target rate
			if (rawRatio >= 1){
				//if 0% to 5% error, just show the target rate
				if (rawRatio <= 1.05){
					rawRate = targetRate;
				}
				//if 5% to 10% error, take the band from 5% to 10% and stretch it
				//to fill the band from 0% to 10%, then add it to the target rate
				else if (rawRatio <= 1.10){
					var scaledError = (rawRatio - 1.05) * 2 * targetRate;
					rawRate = Math.round(targetRate + scaledError);
				}
				else {
					rawRate = Math.round(targetRate * rawRatio);
				}
			}
			//else below the target rate, process same as above
			else {
				if (rawRatio >= 0.95){
					rawRate = targetRate;
				}
				else if (rawRatio >= 0.90){
					var scaledError = (0.95 - rawRatio) * 2 * targetRate;
					rawRate = Math.round(targetRate - scaledError);
				}
				else {
					rawRate = Math.round(targetRate * rawRatio);
				}
			}
		}
		//update the actual rate
		actualRate = rawRate;
	}
	catch (e){
		console.log('Catch error calcActualRate');
	}
}


function calcDoor(){
	try	{
		if (targetRate > 0){
			//calc the drive wheel pulses for 1 roller revolution
			if (gearSetting == 1){
				pulsesPerRollerRev = driveWheelPulsesPerRev * gear1Ratio;
			}
			else if (gearSetting == 2){
				pulsesPerRollerRev = driveWheelPulsesPerRev * gear2Ratio;
			}
			else if (gearSetting == 3){
				pulsesPerRollerRev = driveWheelPulsesPerRev * gear3Ratio;
			}
			else if (gearSetting == 4){
				pulsesPerRollerRev = driveWheelPulsesPerRev * gear4Ratio;
			}
			
			//for each hectare:
			//we can calc the roller revs
			//we know the kg required
			//so we can calc the door height
			var distancePerRollerRev = pulsesPerRollerRev * distancePerPulse;
			var distancePerHa = 10000 / spreadWidth;
			rollerRevsPerHa = distancePerHa / distancePerRollerRev;
			
			//calc our target door height
			var doorHeightInMetres = targetRate / (doorWidth * rollerDiameter * 3.14159265359 * productCal * rollerRevsPerHa);
			targetDoorHeightMm = Math.round(doorHeightInMetres * 1000);
			
			//console.log('calculating door');
			//console.log('spread width: ' + spreadWidth);
			//console.log('distance per ha: ' + distancePerHa);
			//console.log('target rate: ' + targetRate);
			//console.log('roller revs per ha: ' + rollerRevsPerHa);
			//console.log('target door height metres: ' + doorHeightInMetres);
			//console.log('target door height: ' + targetDoorHeightMm);
			
		}
	}
	catch (e){
		console.log('Catch error calcDoor');
	}
}

function setDoor(){
	try	{
		//clear the door kickout alert
		alert6 = false;
		
		//door is 15deg off vertical, this is the equivalent vertical opening
		//cap min/max value before sending to mcu
		if (targetDoorHeightMm < 10){
			targetDoorHeightMv = 10000;
		}
		else if (targetDoorHeightMm > 245){
			targetDoorHeightMv = 0;
		}
		else {
			var targetDoorHeightMv = (targetDoorHeightMm - 10) * 42.553;
			
			targetDoorHeightMv = Math.round(targetDoorHeightMv);
			targetDoorHeightMv = 10000 - targetDoorHeightMv;
		}
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x04;
		txData1[3] = highByte(targetDoorHeightMv);
		txData1[4] = lowByte(targetDoorHeightMv);
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	}
	catch (e){
		console.log('Catch error setDoor');
	}
}


//text for each alert is preset in the run page
function processAlerts(){
	try	{
		//alert1 spinner speed is low
		if (i2cDriveWheelSpeed){
			if (spinnerRpm < 200){
				alert1Counter ++;
			}
			else {
				alert1Counter = 0;
				alert1 = false;
			}
			
			//fire the alert
			if (alert1Counter > 5){
				alert1 = true;
			}
		}
		else {
			alert1Counter = 0;
			alert1 = false;
		}
		
		
		//alert3 output rate not matching the target rate
		if (i2cDriveWheelSpeed && targetRate){
			var rateRatio = actualRate / targetRate;
			
			if (rateRatio < 0.85 || rateRatio > 1.15){
				alert3Counter ++;
			}
			else {
				alert3Counter = 0;
				alert3 = false;
			}
			
			//fire the alert
			if (alert3Counter > 7){
				alert3 = true;
			}
		}
		else {
			alert3Counter = 0;
			alert3 = false;
		}
		
		
		//alert5 target door height is too low
		if (targetDoorHeightMm < 15){
			alert5 = true;
		}
		else {
			alert5 = false;
		}
		
		//alert6 door height problem
		//door kickout flag is self clearing after 1sec
		//this alert remains true until a new door command is sent
		if (actuatorInstalled){
			if (i2cMainMcuFlags & 0x04){
				alert6 = true;
			}
		}
		else {
			alert6 = false;
		}
		
		//alert7 belt slip detected
		
	}
	catch (e){
		console.log('Catch error processAlerts');
	}
}


//concatena bytes and return hex value
function join2bytes(b1, b0){
	var x1 = padByte(b1) + padByte(b0);
	x1 = parseInt(x1, 16);
	return x1;
}

//concatena 3 bytes and return hex value
function join3Bytes(b2, b1, b0){
	var x1 = padByte(b2) + padByte(b1) + padByte(b0);
	x1 = parseInt(x1, 16);
	return x1;
}

//converts byte to string with padded zero
function padByte(b){
	if (b < 0x10){
		b = '0' + b.toString(16);
	}
	else {
		b = b.toString(16);
	}
	return b;
}

//extract low byte
function lowByte(bb){
	var b0 = bb & 0xFF;
	return b0;
}

//extract high byte
function highByte(bb){
	var b1 = (bb >> 8) & 0xFF;
	return b1;
}






//__________________________________________________________________________________________
//server for webpages, sockets, and http requests

var express = require('express');  
var app = express();
var server = require('http').createServer(app);
var io = require('socket.io')(server);

//when express receives a get request, send the index file
app.use(express.static(__dirname)); 
app.get('/', function(request, response, next) 
{	
	console.log('Received http get request');
	
    response.sendFile(__dirname + '/index.html');
});


//socket on, notify when socket is connected
io.on('connection', function(socket){
	
	console.log('Socket is connected');
	
	//simple message from webpage
	socket.on('join', function(data){
		console.log(data);
		socket.emit('status', 'Online');	
	});
	
	//__________________________________________________________________________________________
	//quick set page
	
	//webpage requesting data from the pi
	socket.on('quickSetPage', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled,
			displayWeight: displayWeight,
			hectareCounter: hectareCounter,
			actualRate: actualRate,
			spinnerRpm: spinnerRpm,
			productName: productName,
			alert1: alert1,
			alert2: alert2,
			alert3: alert3,
			alert4: alert4,
			alert5: alert5,
			alert6: alert6,
			alert7: alert7
		});
	});
	
	//__________________________________________________________________________________________
	//products
	
	//webpage requesting data from the pi
	socket.on('products', function(data){
		socket.emit('newData',{
			productName: productName,
			targetRate: targetRate,
			spreadWidth: spreadWidth,
			gearSetting: gearSetting,
			actuatorInstalled: actuatorInstalled,
			targetDoorHeightMm: targetDoorHeightMm,
			productCal: productCal,
			calSampleKg: calSampleKg
		});
	});
	
	//inc the product pointer, reset the timeout to save the config file so
	//we store the active product
	socket.on('nextProduct', function (data){
		
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		productPointer ++;
		if (productPointer > 9){
			productPointer = 0;
		}
		
		productName = storedProducts[productPointer];
		targetRate = storedRates[productPointer];
		spreadWidth = storedWidths[productPointer];
		gearSetting = storedGearSettings[productPointer];
		productCal = storedCals[productPointer];
		calcDoor();
	});
	
	//dec the product pointer, reset the timeout to save the config file so
	//we store the active product
	socket.on('prevProduct', function (data){
		
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		productPointer --;
		if (productPointer < 0){
			productPointer = 9;
		}
		
		productName = storedProducts[productPointer];
		targetRate = storedRates[productPointer];
		spreadWidth = storedWidths[productPointer];
		gearSetting = storedGearSettings[productPointer];
		productCal = storedCals[productPointer];
		calcDoor();
	});
	
	//user updated product info, save it
	socket.on('productSaveName', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		productName = data;
		storedProducts[productPointer] = productName;
	});
	
	//user updated product info, save it
	socket.on('productSaveRate', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		targetRate = data;
		storedRates[productPointer] = targetRate;
		calcDoor();
	});
	
	//user updated product info, save it
	socket.on('productSaveWidth', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		spreadWidth = data;
		storedWidths[productPointer] = spreadWidth;
		calcDoor();
	});
	
	//user updated product info, save it
	socket.on('changeGear', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		if (gearSetting == 4){
			gearSetting = 1;
		}
		else {
			gearSetting ++;
		}
		
		storedGearSettings[productPointer] = gearSetting;
		calcDoor();
	});
	
	//set door to the target height
	socket.on('setDoor', function (data){
		setDoor();
	});
	
	//user updated product info, save it
	socket.on('productSaveCal', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		productCal = data;
		storedCals[productPointer] = productCal;
		calcDoor();
	});
	
	//start new cal sample
	socket.on('newCalSample', function (data){
		//zero the sample, refresh cal factor
		calSampleKg = 0;
		saveDataStorageFile();
		productCal = storedCals[productPointer];
		
	});
	
	//user entered a cal sample weight
	socket.on('calSampleWeight', function (data){
		//if the entered kg and counted kg are valid
		if (data > 0 && calSampleKg > 0){
			clearTimeout(saveConfigTimer);
			saveConfigTimer = setTimeout(saveUserConfig, 4000);
			
			//the correction factor is the entered kg divided by the counted kg
			var calAdjust = data / calSampleKg;
			var adjustedCal = productCal * calAdjust;
			
			//use math round to set 3 decimal places, this method is fastest
			productCal = Math.round(adjustedCal * 1000) / 1000;
			storedCals[productPointer] = productCal;
			
			calcDoor();
		}
		//update to show the entered kg
		calSampleKg = data;
		saveDataStorageFile();
	});
	
	//__________________________________________________________________________________________
	//settings machine
	
	//webpage requesting data from the pi
	socket.on('setupMachine', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled,
			rollerDiameter: rollerDiameter,
			doorWidth: doorWidth,
			driveWheelPulsesPerRev: driveWheelPulsesPerRev,
			distancePerPulse: distancePerPulse,
			i2cDriveWheelSpeed: i2cDriveWheelSpeed,
			actuatorInstalled: actuatorInstalled,
			gear1Ratio: gear1Ratio,
			gear2Ratio: gear2Ratio,
			gear3Ratio: gear3Ratio,
			gear4Ratio: gear4Ratio
		});
	});
	
	//set roller dia
	socket.on('rollerDiaCal', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		rollerDiameter = data;
	});
	
	//set door width
	socket.on('doorWidthCal', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		doorWidth = data;
	});
	
	//set sensor pulses for drive wheel
	socket.on('setWheelPulses', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		driveWheelPulsesPerRev = data;
	});
	
	//wheel sensor cal
	socket.on('wheelCal', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		distancePerPulse = data;
	});
	
	//change actuator installed
	socket.on('changeActuator', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		actuatorInstalled = !actuatorInstalled;
	});
	
	//set gear ratio
	socket.on('saveRatio1', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		gear1Ratio = data;
	});
	
	//set gear ratio
	socket.on('saveRatio2', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		gear2Ratio = data;
	});
	
	//set gear ratio
	socket.on('saveRatio3', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		gear3Ratio = data;
	});
	
	//set gear ratio
	socket.on('saveRatio4', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		gear4Ratio = data;
	});
	
	
	
	//__________________________________________________________________________________________
	//settings device
	
	//webpage requesting data from the pi
	socket.on('setupDevice', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled
		});
	});
	
	//reset to factory settings
	socket.on('factoryReset', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		loadFactoryConfig();
		clearDataStorage();
	});
	
	//exit this app and start servermain
	function reStartApp(){
		var spawn = require('child_process').spawn;
		
		function startApp(){
			spawn('node', ['/home/pi/servermain/servermain'], {
				cwd: '/home/pi/servermain',	//current working directory for child process
				detached: true,				//detatched from the parent process
				stdio: "inherit"			//stay attached to the controlling terminal
			});
		}
		
		process.on('exit', startApp);
		process.exit();
	}
	
	//change app to variable rate
	socket.on('selectAppVR', function(data){
		//load the name of the app to start
		var appName = {
			appName: 'appVR'
		};
		
		//convert to json strings and write to the servermain config file
		var data = JSON.stringify(appName, null, 2);
		fs.writeFileSync('/home/pi/servermain/config.json', data);
		
		setTimeout(reStartApp, 2500);
	});
	
	//change app to fixed rate
	socket.on('selectAppFR', function(data){
		//load the name of the app to start
		var appName = {
			appName: 'appFR'
		};
		
		//convert to json strings and write to the servermain config file
		var data = JSON.stringify(appName, null, 2);
		fs.writeFileSync('/home/pi/servermain/config.json', data);
		
		setTimeout(reStartApp, 2500);
	});
	
	//change app to monitor
	socket.on('selectAppMON', function(data){
		//load the name of the app to start
		var appName = {
			appName: 'appMON'
		};
		
		//convert to json strings and write to the servermain config file
		var data = JSON.stringify(appName, null, 2);
		fs.writeFileSync('/home/pi/servermain/config.json', data);
		
		setTimeout(reStartApp, 2500);
	});
	
	//__________________________________________________________________________________________
	//settings factory
	
	//webpage requesting data from the pi
	socket.on('factoryPage', function(data){
		//nothing required
	});
	
	//save to factory settings
	socket.on('saveFactory', function (data){		
		saveFactoryConfig();
	});
	
	//__________________________________________________________________________________________
	//settings hopper without loadcells
	
	//webpage requesting data from the pi
	socket.on('setupKgCounter', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled,
			quickFillKgs: quickFillKgs
		});
	});
	
	//set hopper kgs
	socket.on('setHopperKg', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		countdownKg = data;
		quickFillKgs = data;
		displayWeight = Math.round(countdownKg / 10) * 10;
		saveDataStorageFile();
	});
	
	//quick fill hopper kgs to previous amount
	socket.on('quickFillHopperKg', function (data){
		countdownKg = quickFillKgs;
		displayWeight = Math.round(countdownKg / 10) * 10;
		saveDataStorageFile();
	});
	
	//enable loadcells
	socket.on('enableLoadCells', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		loadCellsEnabled = true;
		resetFilter = true;
	});
	
	//__________________________________________________________________________________________
	//settings hopper with loadcells
	
	//webpage requesting data from the pi
	socket.on('setupLoadCells', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode,
			displayWeight: displayWeight
		});
	});
	
	//quick zero, set a weight offset
	socket.on('cellQuickZero', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		weightOffset = weightOffset + (0 - displayWeight);
		displayWeight = 0;
	});
	
	//cal zero
	socket.on('cellCalZero', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		weightOffset = 0;
		displayWeight = 0;
		loadCellZero = resolvedLoadCell;
		resetFilter = true;
	});
	
	//cal span
	socket.on('cellCalSpan', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		weightSpan = data;
		loadCellSpan = resolvedLoadCell;
		displayWeight = data;
		resetFilter = true;
	});
	
	//set loadcells to factory cal
	socket.on('factoryLoadCells', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		factoryLoadCells();
	});
	
	//set filter mode
	socket.on('setFilter', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		filterMode ++;
		if (filterMode > 2)
		{
			filterMode = 0;
		}
		
		resetFilter = true;
	});
	
	//set channel on/off
	socket.on('cellChannel0', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell0 = !useLoadCell0;
	});
	
	//set channel on/off
	socket.on('cellChannel1', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell1 = !useLoadCell1;
	});
	
	//set channel on/off
	socket.on('cellChannel2', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell2 = !useLoadCell2;
	});
	
	//set channel on/off
	socket.on('cellChannel3', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell3 = !useLoadCell3;
	});
	
	//disable loadcells
	socket.on('disableLoadCells', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		loadCellsEnabled = false;
	});
	
	
	//__________________________________________________________________________________________
	//settings hectare counter
	
	//webpage requesting data from the pi
	socket.on('setupHectares', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled,
			hectareCounter: hectareCounter
		});
	});
	
	//clear hectare counter
	socket.on('zeroHectares', function (data){
		hectareCounter = 0;
		saveDataStorageFile();
	});
	
	//calibrate hectare counter
	socket.on('calHectares', function (data){
		var newHectares = data;
		if (hectareCounter > 0){
			distancePerPulse = distancePerPulse * (newHectares / hectareCounter);
			
			clearTimeout(saveConfigTimer);
			saveConfigTimer = setTimeout(saveUserConfig, 4000);
		}
		
		hectareCounter = data;
		saveDataStorageFile();
	});
	
	
	//__________________________________________________________________________________________
	//diag system general
	
	//webpage requesting data from the pi
	socket.on('diagGeneral', function(data){
		socket.emit('newData',{
			loadCellMcuOnline: loadCellMcuOnline,
			mainMcuOnline: mainMcuOnline,
			i2cLoadCellCounts: i2cLoadCellCounts,
			i2cMainMcuCounts: i2cMainMcuCounts,
			i2cSupplyVoltage: i2cSupplyVoltage,
			hardwareVer: hardwareVer,
			softwareVer: softwareVer
		});
	});
	
	
	//__________________________________________________________________________________________
	//diag loadcells
	
	//webpage requesting data from the pi
	socket.on('diagLoadCells', function(data){
		socket.emit('newData',{
			i2cLoadCellFlags: i2cLoadCellFlags,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			i2cLoadCell0: i2cLoadCell0,
			i2cLoadCell1: i2cLoadCell1,
			i2cLoadCell2: i2cLoadCell2,
			i2cLoadCell3: i2cLoadCell3,
			resolvedLoadCell: resolvedLoadCell,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode,
			displayWeight: displayWeight
		});
	});
	
	//__________________________________________________________________________________________
	//diag belt
	
	//webpage requesting data from the pi
	socket.on('diagBelt', function(data){
		socket.emit('newData',{
			i2cMainMcuFlags: i2cMainMcuFlags,
			spinnerRpm: spinnerRpm,
			rollerDiameter: rollerDiameter,
			doorWidth: doorWidth,
			distancePerPulse: distancePerPulse,
			gearSetting: gearSetting,
			gear1Ratio: gear1Ratio,
			gear2Ratio: gear2Ratio,
			gear3Ratio: gear3Ratio,
			gear4Ratio: gear4Ratio,
			targetRate: targetRate,
			spreadWidth: spreadWidth,
			productCal: productCal,
			i2cDriveWheelSpeed: i2cDriveWheelSpeed,
			driveWheelPulsesPerRev: driveWheelPulsesPerRev,
			actualRate: actualRate
		});
	});
	
	
	//__________________________________________________________________________________________
	//diag door
	
	//webpage requesting data from the pi
	socket.on('diagDoor', function(data){
		socket.emit('newData',{
			i2cMainMcuFlags: i2cMainMcuFlags,
			i2cDoorSensorMv: i2cDoorSensorMv,
			targetDoorHeightMm: targetDoorHeightMm,
			actualDoorHeightMm: actualDoorHeightMm,
			actuatorInstalled: actuatorInstalled
		});
	});
	
	//set door to the target height
	socket.on('diagSetDoor', function (data){
		setDoor();
	});
	
	//manual move door
	socket.on('openDoor', function (data){
		//clear the door kickout alert
		alert6 = false;
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x08;
		txData1[3] = 0x00;
		txData1[4] = 0x00;
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	});
	
	//manual move door
	socket.on('closeDoor', function (data){
		//clear the door kickout alert
		alert6 = false;
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x09;
		txData1[3] = 0x00;
		txData1[4] = 0x00;
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	});
	
	//stop moving door
	socket.on('stopDoor', function (data){
		//clear the door kickout alert
		alert6 = false;
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x10;
		txData1[3] = 0x00;
		txData1[4] = 0x00;
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	});
	
	
	//__________________________________________________________________________________________


});

//server port
server.listen(80);






