/*
Copyright © 2018 Roesner P/L

i4M Machine Controller - External Rate Controller and Load Cells only
*/

var i2c = require('i2c-bus');
var fs = require('fs');


//log to console if it exists, else set console.log to nop
if (console){
	console.log('Console log enabled');
}
else {
	console.log = function(){};
}

//i2c comms to main board
var i2c1 = i2c.openSync(1);

//buffer for command data to main board
var txData1 = new Buffer(32, 0);

var hardwareVer;
var softwareVer;
var cycleDataTimer;
var cycleCounter = 0;
var saveConfigTimer;

var loadCellMcuOnline = true;
var mainMcuOnline = true;
var i2cLoadCellCounts = 0;
var i2cMainMcuCounts = 0;
var ledMode = 0;

var i2cLoadCellFlags = 0;
var i2cLoadCell0 = 0;
var i2cLoadCell1 = 0;
var i2cLoadCell2 = 0;
var i2cLoadCell3 = 0;
var resolvedLoadCell = 0;
var displayWeight = 0;

var i2cSupplyVoltage = 0;
var i2cSpinnerSpeed = 0;
var i2cPwmInDuty = 0;
var spinnerRpm = 0;

//tracking filter
var deadBand = 0;
var featherTime = 0;
var fineAdjustStart = 0;
var fineAdjustUp = 0;
var fineAdjustDown = 0;
var coarseAdjust = 0;
var tooLowCounter = 0;
var tooHighCounter = 0;

//moving average filter
var sampleBuffer = new Array(100).fill(0);
var numberOfSamples = 100;
var samples = 0;
var samplePointer = 0;
var sampleTotal = 0;
var resetFilter = true;

//config file variables
var showHopperKg;
var showSpinnerRpm;
var useLoadCell0;
var useLoadCell1;
var useLoadCell2;
var useLoadCell3;
var loadCellZero;
var loadCellSpan;
var weightSpan;
var weightOffset;
var filterMode;


readVersionFile();
loadUserConfig();
//loadcell config not required for now, adc reads all 4 channels
//setTimeout(configLoadCellMcu, 1000);
//set controller led to solid green
ledMode = 2;
setTimeout(setLedMode, 1200);
setTimeout(configMainMcu, 1500);
setTimeout(cycleData, 2000);


function readVersionFile(){
	try {
		//read and parse the version file
		var rawData = fs.readFileSync('/home/pi/servermain/version.json');
		var version = JSON.parse(rawData);
		
		hardwareVer = version.hardwareVer;
		softwareVer = version.softwareVer;

		console.log(version);
	}
	catch (e){
		console.log('Catch error readVersionFile');
	}
}

function loadUserConfig(){
	try {
		//read and parse the config file
		var rawData = fs.readFileSync('/home/pi/servermain/appMON/configUser.json');
		var config = JSON.parse(rawData);
		
		showHopperKg = config.showHopperKg;
		showSpinnerRpm = config.showSpinnerRpm;
		useLoadCell0 = config.useLoadCell0;
		useLoadCell1 = config.useLoadCell1;
		useLoadCell2 = config.useLoadCell2;
		useLoadCell3 = config.useLoadCell3;
		loadCellZero = config.loadCellZero;
		loadCellSpan = config.loadCellSpan;
		weightSpan = config.weightSpan;
		weightOffset = config.weightOffset;
		filterMode = config.filterMode;

		console.log('Loading user config');
		console.log(config);
	}
	catch (e){
		//config file is blank or faulty, so load the factory data
		loadFactoryConfig();
		console.log('Catch error loadUserConfig');
	}
}

function loadFactoryConfig(){
	try {
		//read and parse the config file
		var rawData = fs.readFileSync('/home/pi/servermain/appMON/configFactory.json');
		var config = JSON.parse(rawData);
		
		showHopperKg = config.showHopperKg;
		showSpinnerRpm = config.showSpinnerRpm;
		useLoadCell0 = config.useLoadCell0;
		useLoadCell1 = config.useLoadCell1;
		useLoadCell2 = config.useLoadCell2;
		useLoadCell3 = config.useLoadCell3;
		loadCellZero = config.loadCellZero;
		loadCellSpan = config.loadCellSpan;
		weightSpan = config.weightSpan;
		weightOffset = config.weightOffset;
		filterMode = config.filterMode;

		console.log('Loading factory config');
		console.log(config);
	}
	catch (e){
		console.log('Catch error loadFactoryConfig');
	}
}

function factoryLoadCells(){
	try {
		//read and parse the config file
		var rawData = fs.readFileSync('/home/pi/servermain/appMON/configFactory.json');
		var config = JSON.parse(rawData);
		
		useLoadCell0 = config.useLoadCell0;
		useLoadCell1 = config.useLoadCell1;
		useLoadCell2 = config.useLoadCell2;
		useLoadCell3 = config.useLoadCell3;
		loadCellZero = config.loadCellZero;
		loadCellSpan = config.loadCellSpan;
		weightSpan = config.weightSpan;
		weightOffset = config.weightOffset;
		filterMode = config.filterMode;

		console.log('Loading factory loadcells');
		console.log(config);
	}
	catch (e){
		console.log('Catch error loadFactoryLoadCell');
	}
}

function saveUserConfig(){
	try {
		//load variables used by pi and main board
		var configData = {
			showHopperKg: showHopperKg,
			showSpinnerRpm: showSpinnerRpm,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode
		};
		
		//convert to json strings and write to file
		var data = JSON.stringify(configData, null, 2);
		fs.writeFileSync('/home/pi/servermain/appMON/configUser.json', data);
		
		console.log('Saving user config file');
	}
	catch (e){
		console.log('Catch error saveUserConfig');
	}
}

function saveFactoryConfig(){
	try {
		//load variables used by pi and main board
		var configData = {
			showHopperKg: showHopperKg,
			showSpinnerRpm: showSpinnerRpm,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode
		};
		
		//convert to json strings and write to file
		var data = JSON.stringify(configData, null, 2);
		fs.writeFileSync('/home/pi/servermain/appMON/configFactory.json', data);
		
		console.log('Saving factory config file');
	}
	catch (e){
		console.log('Catch error saveFactoryConfig');
	}
}

function setLedMode(){
	try {
		//led modes 1 = solid red, 2 = solid green, 3 = flashing red, 4 = flashing green
		//command bytes to send via i2c
		txData1[0] = 0xc1;
		txData1[1] = 0x00;
		txData1[2] = 0x03;
		txData1[3] = ledMode;
		txData1[4] = 0x00;
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	}
	catch (e){
		console.log('Catch error setLedMode');
	}
}

function configLoadCellMcu(){
	try {
		var configFlags = 0;
		if (useLoadCell0 == true){
			configFlags = configFlags | 0x01;
		}
		if (useLoadCell1 == true){
			configFlags = configFlags | 0x02;
		}
		if (useLoadCell2 == true){
			configFlags = configFlags | 0x04;
		}
		if (useLoadCell3 == true){
			configFlags = configFlags | 0x08;
		}
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x02;
		txData1[3] = 0x00;
		txData1[4] = lowByte(configFlags);
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to loadcell mcu
		writeLoadCellMcu(txData1);
	}
	catch (e){
		console.log('Catch error configLoadCellMcu');
	}
}


function configMainMcu(){
	try {
		//not required
	}
	catch (e){
		console.log('Catch error configMainMcu');
	}
}

//called regularly to get data from the main board
function cycleData(){
	try {
		//if the slave is online, read it
		if (loadCellMcuOnline){
			readLoadCellMcu();
		}
		
		//allow time for mainboard to respond min 5ms
		setTimeout(processLoadCells, 30);
		
		//we read loadcells more often than the main mcu (for the filter)
		//so after some number of loadcell reads, read the main mcu
		cycleCounter ++;
		
		if (cycleCounter == 5){
			cycleCounter = 0;
			if (mainMcuOnline){
				readMainMcu();
				//allow time for mainboard to respond min 5ms
				setTimeout(processMainMcu, 30);
			}
		}
		
		//run again after this delay
		cycleDataTimer = setTimeout(cycleData, 200);
	}
	catch (e){
		console.log('Catch error cycleData');
	}
}

function readLoadCellMcu(){
	try {
		//message settings
		var slaveID = 80;
		var slaveRegister = 100;
		var packetLength = 15;
		var packet = new Buffer(32, 0);
		
		//read block of bytes
		i2c1.readI2cBlockSync(slaveID, slaveRegister, packetLength, packet);
		
		//if valid end of frame byte
		//clr failure counter, and transfer the data
		if (packet[14] == 0xC3){
			loadCellMcuOnline = true;
			i2cLoadCellCounts ++;
			
			console.log('I2C mcu2 read loadcells:   OK   ' + i2cLoadCellCounts);
			
			i2cLoadCellFlags = packet[1];
			i2cLoadCell0 = join3Bytes(packet[2], packet[3], packet[4]);
			i2cLoadCell1 = join3Bytes(packet[5], packet[6], packet[7]);
			i2cLoadCell2 = join3Bytes(packet[8], packet[9], packet[10]);
			i2cLoadCell3 = join3Bytes(packet[11], packet[12], packet[13]);
		}
		else {
			console.log('End of frame error readLoadCellMcu');
			//reset this flag after 2secs, this resets the mcu i2c module
			loadCellMcuOnline = false;
			setTimeout(function(){ loadCellMcuOnline = true; }, 2000);
		}
	}
	catch (e){
		console.log('Catch error readLoadCellMcu');
		//reset this flag after 2secs, this resets the mcu i2c module
		loadCellMcuOnline = false;
		setTimeout(function(){ loadCellMcuOnline = true; }, 2000);
	}
}



var scaledCell0 = 0;
var scaledCell1 = 0;
var scaledCell2 = 0;
var scaledCell3 = 0;


function processLoadCells(){
	try	{
		//sum the active loadcell channels and div by number of channels
		var average = 0;
		var channels = 0;
		var tempDisplayWeight = displayWeight;
		
		//hard coded scale factor for chaser bin
		//drawbar is 5x more sensitive than axle cells
		//so mul axle cells by 5 to match drabar sensitivity
		//cell0 is axle
		//cell1 is axle
		//cell2 is drawbar
		//cell3 is unused
		
		var tempVar = 0;
		
		//65536 is 0mV, so sub it from i2cLoadCell value
		//scale the result as required then add it back to 65536
		tempVar = i2cLoadCell0 - 65536;
		tempVar = tempVar * 5;
		scaledCell0 = tempVar + 65536;
		
		tempVar = i2cLoadCell1 - 65536;
		tempVar = tempVar * 5;
		scaledCell1 = tempVar + 65536;
		
		
		//don't scale these channels
		scaledCell2 = i2cLoadCell2;
		scaledCell3 = i2cLoadCell3;
		
		
		if (useLoadCell0 == true){
			average += scaledCell0;
			channels ++;
		}
		if (useLoadCell1 == true){
			average += scaledCell1;
			channels ++;
		}
		if (useLoadCell2 == true){
			average += scaledCell2;
			channels ++;
		}
		if (useLoadCell3 == true){
			average += scaledCell3;
			channels ++;
		}
		
		average = Math.round(average / channels);
		resolvedLoadCell = average;
		
		//if loadcell range is not zero, calc the weight
		if (loadCellSpan - loadCellZero){
			
			var newWeight = weightSpan * (resolvedLoadCell - loadCellZero);
			newWeight = newWeight / (loadCellSpan - loadCellZero);
			newWeight = parseInt(newWeight, 10);
			
			//apply the weight offset now
			newWeight += weightOffset;
			
			if (resetFilter){
				//reset data for moving ave filter
				sampleBuffer.fill(0);
				samples = 0;
				samplePointer = 0;
				sampleTotal = 0;
				resetFilter = false;
			}
			
			switch (filterMode){
				
				//filter off
				case 0:
					tempDisplayWeight = newWeight;
				break;
				
				
				//tracking filter, if display is too low, it will gradually increase
				//until feather time elapses, then apply coarse adjust
				case 1:
					featherTime = 15;
					deadBand = 5;
					fineAdjustStart = 4;
					fineAdjustUp += 2;
					fineAdjustDown += 2;
					coarseAdjust = 200;
					
					//too high
					if (displayWeight > (newWeight + deadBand))
					{
						tooHighCounter ++;
						tooLowCounter = 0;
						fineAdjustUp = 0;
						
						if (tooHighCounter < featherTime)
						{	
							tempDisplayWeight -= fineAdjustStart + fineAdjustDown;
						}
						else
						{
							tempDisplayWeight -= coarseAdjust;
							fineAdjustDown = 0;
							
							if (tempDisplayWeight < newWeight)
							{
								tempDisplayWeight = newWeight;
								tooHighCounter = 0;
							}
						}
					}
					//too low
					else if (displayWeight < (newWeight - deadBand))
					{
						tooLowCounter ++;
						tooHighCounter = 0;
						fineAdjustDown = 0;
						
						if (tooLowCounter < featherTime)
						{
							tempDisplayWeight += fineAdjustStart + fineAdjustUp;
						}
						else
						{
							tempDisplayWeight += coarseAdjust;
							fineAdjustUp = 0;
							
							if (tempDisplayWeight > newWeight)
							{
								tempDisplayWeight = newWeight;
								tooLowCounter = 0;
							}
						}
					}
					else
					{
						tooHighCounter = 0;
						tooLowCounter = 0;
						fineAdjustUp = 0;
						fineAdjustDown = 0;
					}
				break;
				
				
				//moving average filter
				case 2:
					numberOfSamples = 50;
					
					if (samples < numberOfSamples)
					{
						samples ++;
					}
					
					sampleTotal -= sampleBuffer[samplePointer];
					sampleTotal += newWeight;
					sampleBuffer[samplePointer] = newWeight;
					
					tempDisplayWeight = sampleTotal / samples;
					
					samplePointer ++;
					
					if (samplePointer == numberOfSamples)
					{
						samplePointer = 0;
					}
				break;
				
				default:
			}
			
			//if filter is on, snap to zero if within 50kg
			if (filterMode){
				if (tempDisplayWeight < 50 && tempDisplayWeight > -50){
					tempDisplayWeight = 0;
				}
			}
			
		}
		else {
			tempDisplayWeight = 0;
		}
		
		//update display, round to 1kg
		//displayWeight = Math.round(tempDisplayWeight);
		
		//if zero or span is the default value, load a big negative number
		//so the display prompts user to calibrate
		if (loadCellZero == 0 || loadCellSpan == 130000){
			displayWeight = -100000;
		}
		else {
			//update display, round to 10kg
			displayWeight = Math.round(tempDisplayWeight / 10) * 10;
		}
		
	}
	catch (e){
		console.log('Catch error processLoadCells');
	}
}

function writeLoadCellMcu(txData){
	try	{
		//message settings
		var slaveID = 80;
		var slaveRegister = 0;
		var packetLength = 15;
		
		//load end of frame byte
		txData[14] = 0xC3;
		
		//send block of bytes
		i2c1.writeI2cBlockSync(slaveID, slaveRegister, packetLength, txData);
		
		console.log('I2C loadcell mcu:          WRITE');
		
		//i2c1.closeSync();
	}
	catch (e){
		console.log('Catch error writeLoadCellMcu');
	}
}

function writeMainMcu(txData){
	try	{
		//message settings
		var slaveID = 60;
		var slaveRegister = 0;
		var packetLength = 15;
		
		//load end of frame byte
		txData[14] = 0xC3;
		
		//send block of bytes
		i2c1.writeI2cBlockSync(slaveID, slaveRegister, packetLength, txData);
		
		console.log('I2C main mcu:              WRITE');
		
		//i2c1.closeSync();
	}
	catch (e){
		console.log('Catch error writeMainMcu');
	}
}

function readMainMcu(){
	try	{
		//message settings
		var slaveID = 60;
		var slaveRegister = 100;
		var packetLength = 30;
		var packet = new Buffer(32, 0);
		
		//read block of bytes
		i2c1.readI2cBlockSync(slaveID, slaveRegister, packetLength, packet);
		
		//if valid end of frame byte
		//clr failure counter, and transfer the data
		if (packet[29] == 0xC3){
			mainMcuOnline = true;
			i2cMainMcuCounts ++;
				
			console.log('I2C mcu1 read data1:       OK   ' + i2cMainMcuCounts);
			
			i2cSupplyVoltage = packet[3];
			i2cSpinnerSpeed = join2bytes(packet[6], packet[7]);
			i2cPwmInDuty = packet[15];
		}
		else {
			console.log('End of frame error readMainMcu');
			//reset this flag after 2secs, this resets the mcu i2c module
			mainMcuOnline = false;
			setTimeout(function(){ mainMcuOnline = true; }, 2000);
		}
	}
	catch (e){
		console.log('Catch error readMainMcu');
		//reset this flag after 2secs, this resets the mcu i2c module
		mainMcuOnline = false;
		setTimeout(function(){ mainMcuOnline = true; }, 2000);
	}
}

function processMainMcu(){
	try {
		//spinner speed rpm rounded to nearest 10
		spinnerRpm = Math.round(i2cSpinnerSpeed * 0.06) * 10;
		
	}
	catch (e){
		console.log('Catch error processMainMcu');
	}
}

//concatena bytes and return hex value
function join2bytes(b1, b0){
	var x1 = padByte(b1) + padByte(b0);
	x1 = parseInt(x1, 16);
	return x1;
}

//concatena 3 bytes and return hex value
function join3Bytes(b2, b1, b0){
	var x1 = padByte(b2) + padByte(b1) + padByte(b0);
	x1 = parseInt(x1, 16);
	return x1;
}

//converts byte to string with padded zero
function padByte(b){
	if (b < 0x10){
		b = '0' + b.toString(16);
	}
	else {
		b = b.toString(16);
	}
	return b;
}

//extract low byte
function lowByte(bb){
	var b0 = bb & 0xFF;
	return b0;
}

//extract high byte
function highByte(bb){
	var b1 = (bb >> 8) & 0xFF;
	return b1;
}






//__________________________________________________________________________________________
//server for webpages, sockets, and http requests

var express = require('express');  
var app = express();
var server = require('http').createServer(app);
var io = require('socket.io')(server);

//when express receives a get request, send the index file
app.use(express.static(__dirname)); 
app.get('/', function(request, response, next) 
{	
	console.log('Received http get request');
	
    response.sendFile(__dirname + '/index.html');
});

//when express receives a post request, extract the data and respond
app.use(express.json());
app.use(express.urlencoded({ extended: true })); 
app.post('/', function(request, response, next) 
{	
	var bodyData = request.body;	

	if(bodyData.command == 'refreshData'){

		console.log("Received data request from app");	

		// Send new data to tablet
		response.send({
			displayWeight: displayWeight
		});		
		
	}

});


//socket on, notify when socket is connected
io.on('connection', function(socket){
	
	console.log('Socket is connected');
	
	//simple message from webpage
	socket.on('join', function(data){
		console.log(data);
		socket.emit('status', 'Online');	
	});
	
	//__________________________________________________________________________________________
	//quick set page
	
	//webpage requesting data from the pi
	socket.on('quickSetPage', function(data){
		socket.emit('newData',{
			showHopperKg: showHopperKg,
			showSpinnerRpm: showSpinnerRpm,
			displayWeight: displayWeight,
			spinnerRpm: spinnerRpm
		});
	});
	
	//__________________________________________________________________________________________
	//settings machine
	
	//webpage requesting data from the pi
	socket.on('setupMachine', function(data){
		socket.emit('newData',{
			showHopperKg: showHopperKg,
			showSpinnerRpm: showSpinnerRpm
		});
	});
	
	//show hide hopper readout
	socket.on('hopperOnOff', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		showHopperKg = !showHopperKg;
	});
	
	//show hide spinner readout
	socket.on('spinnerOnOff', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		showSpinnerRpm = !showSpinnerRpm;
	});
	
	
	//__________________________________________________________________________________________
	//settings device
	
	//webpage requesting data from the pi
	socket.on('setupDevice', function(data){
		socket.emit('newData',{
			
		});
	});
	
	//reset to factory settings
	socket.on('factoryReset', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		loadFactoryConfig();
	});
	
	//exit this app and start servermain
	function reStartApp(){
		var spawn = require('child_process').spawn;
		
		function startApp(){
			spawn('node', ['/home/pi/servermain/servermain'], {
				cwd: '/home/pi/servermain',	//current working directory for child process
				detached: true,				//detatched from the parent process
				stdio: "inherit"			//stay attached to the controlling terminal
			});
		}
		
		process.on('exit', startApp);
		process.exit();
	}
	
	//change app to variable rate
	socket.on('selectAppVR', function(data){
		//load the name of the app to start
		var appName = {
			appName: 'appVR'
		};
		
		//convert to json strings and write to the servermain config file
		var data = JSON.stringify(appName, null, 2);
		fs.writeFileSync('/home/pi/servermain/config.json', data);
		
		setTimeout(reStartApp, 2500);
	});
	
	//change app to fixed rate
	socket.on('selectAppFR', function(data){
		//load the name of the app to start
		var appName = {
			appName: 'appFR'
		};
		
		//convert to json strings and write to the servermain config file
		var data = JSON.stringify(appName, null, 2);
		fs.writeFileSync('/home/pi/servermain/config.json', data);
		
		setTimeout(reStartApp, 2500);
	});
	
	//change app to monitor
	socket.on('selectAppMON', function(data){
		//load the name of the app to start
		var appName = {
			appName: 'appMON'
		};
		
		//convert to json strings and write to the servermain config file
		var data = JSON.stringify(appName, null, 2);
		fs.writeFileSync('/home/pi/servermain/config.json', data);
		
		setTimeout(reStartApp, 2500);
	});
	
	//__________________________________________________________________________________________
	//settings factory
	
	//webpage requesting data from the pi
	socket.on('factoryPage', function(data){
		//nothing required
	});
	
	//save to factory settings
	socket.on('saveFactory', function (data){		
		saveFactoryConfig();
	});
	
	//__________________________________________________________________________________________
	//settings loadcells
	
	//webpage requesting data from the pi
	socket.on('setupLoadCells', function(data){
		socket.emit('newData',{
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode,
			displayWeight: displayWeight
		});
	});
	
	//quick zero, set a weight offset
	socket.on('cellQuickZero', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		weightOffset = weightOffset + (0 - displayWeight);
		displayWeight = 0;
	});
	
	//cal zero
	socket.on('cellCalZero', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		weightOffset = 0;
		displayWeight = 0;
		loadCellZero = resolvedLoadCell;
		resetFilter = true;
	});
	
	//cal span
	socket.on('cellCalSpan', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		weightSpan = data;
		loadCellSpan = resolvedLoadCell;
		displayWeight = data;
		resetFilter = true;
	});
	
	//set loadcells to factory cal
	socket.on('factoryLoadCells', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		factoryLoadCells();
	});
	
	//set filter mode
	socket.on('setFilter', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		filterMode ++;
		if (filterMode > 2)
		{
			filterMode = 0;
		}
		
		resetFilter = true;
	});
	
	//set channel on/off
	socket.on('cellChannel0', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell0 = !useLoadCell0;
	});
	
	//set channel on/off
	socket.on('cellChannel1', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell1 = !useLoadCell1;
	});
	
	//set channel on/off
	socket.on('cellChannel2', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell2 = !useLoadCell2;
	});
	
	//set channel on/off
	socket.on('cellChannel3', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell3 = !useLoadCell3;
	});
	
	//__________________________________________________________________________________________
	//diag system general
	
	//webpage requesting data from the pi
	socket.on('diagGeneral', function(data){
		socket.emit('newData',{
			loadCellMcuOnline: loadCellMcuOnline,
			mainMcuOnline: mainMcuOnline,
			i2cLoadCellCounts: i2cLoadCellCounts,
			i2cMainMcuCounts: i2cMainMcuCounts,
			i2cSupplyVoltage: i2cSupplyVoltage,
			hardwareVer: hardwareVer,
			softwareVer: softwareVer
		});
	});
	
	
	//__________________________________________________________________________________________
	//diag loadcells
	
	//webpage requesting data from the pi
	socket.on('diagLoadCells', function(data){
		socket.emit('newData',{
			i2cLoadCellFlags: i2cLoadCellFlags,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			i2cLoadCell0: i2cLoadCell0,
			i2cLoadCell1: i2cLoadCell1,
			i2cLoadCell2: i2cLoadCell2,
			i2cLoadCell3: i2cLoadCell3,
			scaledCell0: scaledCell0,
			scaledCell1: scaledCell1,
			scaledCell2: scaledCell2,
			scaledCell3: scaledCell3,
			resolvedLoadCell: resolvedLoadCell,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode,
			displayWeight: displayWeight
		});
	});
	
	//__________________________________________________________________________________________
	//diag sensors
	
	//webpage requesting data from the pi
	socket.on('diagSensors', function(data){
		socket.emit('newData',{
			i2cPwmInDuty: i2cPwmInDuty,
			spinnerRpm: spinnerRpm
		});
	});
	
	//__________________________________________________________________________________________


});

//server port
server.listen(80);






