/*
Copyright © 2018 Roesner P/L

i4M Machine Controller
*/

var i2c = require('i2c-bus');
var fs = require('fs');


//log to console if it exists, else set console.log to nop
if (console){
	console.log('Console log enabled');
}
else {
	console.log = function(){};
}

//i2c comms to main board
var i2c1 = i2c.openSync(1);

//buffer for command data to main board
var txData1 = new Buffer(32, 0);

var hardwareVer;
var softwareVer;
var cycleDataTimer;
var cycleCounter = 0;
var saveConfigTimer;
var saveDataTimer;

var loadCellMcuOnline = true;
var mainMcuOnline = true;
var i2cLoadCellCounts = 0;
var i2cMainMcuCounts = 0;
var ledMode = 0;

var i2cLoadCellFlags = 0;
var i2cLoadCell0 = 0;
var i2cLoadCell1 = 0;
var i2cLoadCell2 = 0;
var i2cLoadCell3 = 0;
var resolvedLoadCell = 0;
var displayWeight = 0;

var i2cMainMcuFlags = 0;
var i2cSupplyVoltage = 0;
var i2cBeltSpeed = 0;
var i2cSpinnerSpeed = 0;
var i2cGroundSpeed = 0;
var i2cDoorSensorMv = 0;
var i2cValve1Command = 0;
var i2cTargetBeltSpeed = 0;
var i2cPwmInDuty = 0;

var masterOn = false;
var i2cMasterFlipFlop = 0;
var targetRate = 0;
var actualRate = 0;
var rawRate = 0;
var spinnerRpm = 0;
var simGroundSpeedPps = 0;
var simGroundSpeedKph = 0;
var manualValve1 = 0;
var manualValve2 = 0;
var targetDoorHeightMv = 0;
var actualDoorHeightMm = 0;

var beltSpeedRatio = 0;
var kgPerRevolution = 0;
var pulsesPerKg = 0;
var pulsesPerHa = 0;

var instantHectares = 0;
var alert1 = false;
var alert2 = false;
var alert3 = false;
var alert4 = false;
var alert5 = false;
var alert6 = false;
var alert7 = false;
var alert1Counter = 0;
var alert2Counter = 0;
var alert3Counter = 0;
var alert4Counter = 0;
var alert5Counter = 0;
var alert6Counter = 0;
var alert7Counter = 0;

//tracking filter
var deadBand = 0;
var featherTime = 0;
var fineAdjustStart = 0;
var fineAdjustUp = 0;
var fineAdjustDown = 0;
var coarseAdjust = 0;
var tooLowCounter = 0;
var tooHighCounter = 0;

//moving average filter
var sampleBuffer = new Array(100).fill(0);
var numberOfSamples = 100;
var samples = 0;
var samplePointer = 0;
var sampleTotal = 0;
var resetFilter = true;

//data file variables
var countdownKg = 0;
var calSampleKg = 0;
var hectareCounter = 0;

//config file variables
var loadCellsEnabled;
var quickFillKgs;
var useLoadCell0;
var useLoadCell1;
var useLoadCell2;
var useLoadCell3;
var loadCellZero;
var loadCellSpan;
var weightSpan;
var weightOffset;
var filterMode;
var doorWidth;
var rollerDiameter;
var rollerPulsesPerRev;
var valve1JumpStart;
var valve1Speed;
var valve1Brake;
var valve1TargetSize;
var distancePerPulse;
var actuatorInstalled;
var storedProducts;
var storedCals;
var storedWidths;
var storedDoorHeights;
var productPointer;
var productName;
var productCal;
var spreadWidth;
var targetDoorHeightMm;

readVersionFile();
readDataStorageFile();
loadUserConfig();
//loadcell config not required for now, adc reads all 4 channels
//setTimeout(configLoadCellMcu, 1000);
//set controller led to solid green
ledMode = 2;
setTimeout(setLedMode, 1200);
setTimeout(configMainMcu, 1500);
setTimeout(cycleData, 2000);


function readVersionFile(){
	try {
		//read and parse the version file
		var rawData = fs.readFileSync('/home/pi/servermain/version.json');
		var version = JSON.parse(rawData);
		
		hardwareVer = version.hardwareVer;
		softwareVer = version.softwareVer;

		console.log(version);
	}
	catch (e){
		console.log('Catch error readVersionFile');
	}
}

function readDataStorageFile(){
	try {
		//read and parse the dataStorage file
		var rawData = fs.readFileSync('/home/pi/servermain/appVR/dataStorage.json');
		var dataStorage = JSON.parse(rawData);
		
		countdownKg = dataStorage.countdownKg;
		calSampleKg = dataStorage.calSampleKg;
		hectareCounter = dataStorage.hectareCounter;
	}
	catch (e){
		console.log('Catch error readDataStorageFile');
	}
}

function saveDataStorageFile(){
	try {
		//load variables to save to dataStorage
		var dataStorage = {
			countdownKg: countdownKg,
			calSampleKg: calSampleKg,
			hectareCounter: hectareCounter
		};
		
		//convert to json strings and write to file
		var data = JSON.stringify(dataStorage, null, 2);
		fs.writeFileSync('/home/pi/servermain/appVR/dataStorage.json', data);
		
		console.log('Saving dataStorage file');
	}
	catch (e){
		console.log('Catch error saveDataStorageFile');
	}
}

function clearDataStorage(){
	try {
		//clear stored data
		countdownKg = 0;
		calSampleKg = 0;
		hectareCounter = 0;
		
		saveDataStorageFile();
	}
	catch (e){
		console.log('Catch error clearDataStorage');
	}
}

function loadUserConfig(){
	try {
		//read and parse the config file
		var rawData = fs.readFileSync('/home/pi/servermain/appVR/configUser.json');
		var config = JSON.parse(rawData);
		
		loadCellsEnabled = config.loadCellsEnabled;
		quickFillKgs = config.quickFillKgs;
		useLoadCell0 = config.useLoadCell0;
		useLoadCell1 = config.useLoadCell1;
		useLoadCell2 = config.useLoadCell2;
		useLoadCell3 = config.useLoadCell3;
		loadCellZero = config.loadCellZero;
		loadCellSpan = config.loadCellSpan;
		weightSpan = config.weightSpan;
		weightOffset = config.weightOffset;
		filterMode = config.filterMode;
		doorWidth = config.doorWidth;
		rollerDiameter = config.rollerDiameter;
		rollerPulsesPerRev = config.rollerPulsesPerRev;
		valve1JumpStart = config.valve1JumpStart;
		valve1Speed = config.valve1Speed;
		valve1Brake = config.valve1Brake;
		valve1TargetSize = config.valve1TargetSize;
		distancePerPulse = config.distancePerPulse;
		actuatorInstalled = config.actuatorInstalled;
		storedProducts = config.storedProducts;
		storedCals = config.storedCals;
		storedWidths = config.storedWidths;
		storedDoorHeights = config.storedDoorHeights;
		productPointer = config.productPointer;
		
		//set product variables based on the stored pointer
		productName = storedProducts[productPointer];
		productCal = storedCals[productPointer];
		spreadWidth = storedWidths[productPointer];
		targetDoorHeightMm = storedDoorHeights[productPointer];

		console.log('Loading user config');
		console.log(config);
	}
	catch (e){
		//config file is blank or faulty, so load the factory data
		loadFactoryConfig();
		console.log('Catch error loadUserConfig');
	}
}

function loadFactoryConfig(){
	try {
		//read and parse the config file
		var rawData = fs.readFileSync('/home/pi/servermain/appVR/configFactory.json');
		var config = JSON.parse(rawData);
		
		loadCellsEnabled = config.loadCellsEnabled;
		quickFillKgs = config.quickFillKgs;
		useLoadCell0 = config.useLoadCell0;
		useLoadCell1 = config.useLoadCell1;
		useLoadCell2 = config.useLoadCell2;
		useLoadCell3 = config.useLoadCell3;
		loadCellZero = config.loadCellZero;
		loadCellSpan = config.loadCellSpan;
		weightSpan = config.weightSpan;
		weightOffset = config.weightOffset;
		filterMode = config.filterMode;
		doorWidth = config.doorWidth;
		rollerDiameter = config.rollerDiameter;
		rollerPulsesPerRev = config.rollerPulsesPerRev;
		valve1JumpStart = config.valve1JumpStart;
		valve1Speed = config.valve1Speed;
		valve1Brake = config.valve1Brake;
		valve1TargetSize = config.valve1TargetSize;
		distancePerPulse = config.distancePerPulse;
		actuatorInstalled = config.actuatorInstalled;
		storedProducts = config.storedProducts;
		storedCals = config.storedCals;
		storedWidths = config.storedWidths;
		storedDoorHeights = config.storedDoorHeights;
		productPointer = config.productPointer;
		
		//set product variables based on the stored pointer
		productName = storedProducts[productPointer];
		productCal = storedCals[productPointer];
		spreadWidth = storedWidths[productPointer];
		targetDoorHeightMm = storedDoorHeights[productPointer];

		console.log('Loading factory config');
		console.log(config);
	}
	catch (e){
		console.log('Catch error loadFactoryConfig');
	}
}

function factoryLoadCells(){
	try {
		//read and parse the config file
		var rawData = fs.readFileSync('/home/pi/servermain/appMON/configFactory.json');
		var config = JSON.parse(rawData);
		
		useLoadCell0 = config.useLoadCell0;
		useLoadCell1 = config.useLoadCell1;
		useLoadCell2 = config.useLoadCell2;
		useLoadCell3 = config.useLoadCell3;
		loadCellZero = config.loadCellZero;
		loadCellSpan = config.loadCellSpan;
		weightSpan = config.weightSpan;
		weightOffset = config.weightOffset;
		filterMode = config.filterMode;

		console.log('Loading factory loadcells');
		console.log(config);
	}
	catch (e){
		console.log('Catch error loadFactoryLoadCell');
	}
}

function saveUserConfig(){
	try {
		//load variables used by pi and main board
		var configData = {
			loadCellsEnabled: loadCellsEnabled,
			quickFillKgs: quickFillKgs,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode,
			doorWidth: doorWidth,
			rollerDiameter: rollerDiameter,
			rollerPulsesPerRev: rollerPulsesPerRev,
			valve1JumpStart: valve1JumpStart,
			valve1Speed: valve1Speed,
			valve1Brake: valve1Brake,
			valve1TargetSize: valve1TargetSize,
			distancePerPulse: distancePerPulse,
			actuatorInstalled: actuatorInstalled,
			storedProducts: storedProducts,
			storedCals: storedCals,
			storedWidths: storedWidths,
			storedDoorHeights: storedDoorHeights,
			productPointer: productPointer
		};
		
		//convert to json strings and write to file
		var data = JSON.stringify(configData, null, 2);
		fs.writeFileSync('/home/pi/servermain/appVR/configUser.json', data);
		
		console.log('Saving user config file');
	}
	catch (e){
		console.log('Catch error saveUserConfig');
	}
}

function saveFactoryConfig(){
	try {
		//load variables used by pi and main board
		var configData = {
			loadCellsEnabled: loadCellsEnabled,
			quickFillKgs: quickFillKgs,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode,
			doorWidth: doorWidth,
			rollerDiameter: rollerDiameter,
			rollerPulsesPerRev: rollerPulsesPerRev,
			valve1JumpStart: valve1JumpStart,
			valve1Speed: valve1Speed,
			valve1Brake: valve1Brake,
			valve1TargetSize: valve1TargetSize,
			distancePerPulse: distancePerPulse,
			actuatorInstalled: actuatorInstalled,
			storedProducts: storedProducts,
			storedCals: storedCals,
			storedWidths: storedWidths,
			storedDoorHeights: storedDoorHeights,
			productPointer: productPointer
		};
		
		//convert to json strings and write to file
		var data = JSON.stringify(configData, null, 2);
		fs.writeFileSync('/home/pi/servermain/appVR/configFactory.json', data);
		
		console.log('Saving factory config file');
	}
	catch (e){
		console.log('Catch error saveFactoryConfig');
	}
}

function setLedMode(){
	try {
		//led modes 1 = solid red, 2 = solid green, 3 = flashing red, 4 = flashing green
		//command bytes to send via i2c
		txData1[0] = 0xc1;
		txData1[1] = 0x00;
		txData1[2] = 0x03;
		txData1[3] = ledMode;
		txData1[4] = 0x00;
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	}
	catch (e){
		console.log('Catch error setLedMode');
	}
}

function configLoadCellMcu(){
	try {
		var configFlags = 0;
		if (useLoadCell0 == true){
			configFlags = configFlags | 0x01;
		}
		if (useLoadCell1 == true){
			configFlags = configFlags | 0x02;
		}
		if (useLoadCell2 == true){
			configFlags = configFlags | 0x04;
		}
		if (useLoadCell3 == true){
			configFlags = configFlags | 0x08;
		}
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x02;
		txData1[3] = 0x00;
		txData1[4] = lowByte(configFlags);
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to loadcell mcu
		writeLoadCellMcu(txData1);
	}
	catch (e){
		console.log('Catch error configLoadCellMcu');
	}
}


function configMainMcu(){
	try {
		//not required
	}
	catch (e){
		console.log('Catch error configMainMcu');
	}
}

//called regularly to get data from the main board
function cycleData(){
	try {
		//if the slave is online, read it
		if (loadCellMcuOnline){
			readLoadCellMcu();
		}
		
		//process loadcells if they are enabled
		//allow time for mainboard to respond min 5ms
		if (loadCellsEnabled){
			setTimeout(processLoadCells, 30);
		}
		
		//we read loadcells more often than the main mcu (for the filter)
		//so after some number of loadcell reads, read the main mcu
		cycleCounter ++;
		
		if (cycleCounter == 5){
			cycleCounter = 0;
			if (mainMcuOnline){
				readMainMcu();
				//allow time for mainboard to respond min 5ms
				setTimeout(processMainMcu, 30);
			}
		}
		
		//run again after this delay
		cycleDataTimer = setTimeout(cycleData, 200);
	}
	catch (e){
		console.log('Catch error cycleData');
	}
}

function readLoadCellMcu(){
	try {
		//message settings
		var slaveID = 80;
		var slaveRegister = 100;
		var packetLength = 15;
		var packet = new Buffer(32, 0);
		
		//read block of bytes
		i2c1.readI2cBlockSync(slaveID, slaveRegister, packetLength, packet);
		
		//if valid end of frame byte
		//clr failure counter, and transfer the data
		if (packet[14] == 0xC3){
			loadCellMcuOnline = true;
			i2cLoadCellCounts ++;
			
			console.log('I2C mcu2 read loadcells:   OK   ' + i2cLoadCellCounts);
			
			i2cLoadCellFlags = packet[1];
			i2cLoadCell0 = join3Bytes(packet[2], packet[3], packet[4]);
			i2cLoadCell1 = join3Bytes(packet[5], packet[6], packet[7]);
			i2cLoadCell2 = join3Bytes(packet[8], packet[9], packet[10]);
			i2cLoadCell3 = join3Bytes(packet[11], packet[12], packet[13]);
		}
		else {
			console.log('End of frame error readLoadCellMcu');
			//reset this flag after 2secs, this resets the mcu i2c module
			loadCellMcuOnline = false;
			setTimeout(function(){ loadCellMcuOnline = true; }, 2000);
		}
	}
	catch (e){
		console.log('Catch error readLoadCellMcu');
		//reset this flag after 2secs, this resets the mcu i2c module
		loadCellMcuOnline = false;
		setTimeout(function(){ loadCellMcuOnline = true; }, 2000);
	}
}

function processLoadCells(){
	try	{
		//sum the active loadcell channels and div by number of channels
		var average = 0;
		var channels = 0;
		var tempDisplayWeight = displayWeight;
		
		if (useLoadCell0 == true){
			average += i2cLoadCell0;
			channels ++;
		}
		if (useLoadCell1 == true){
			average += i2cLoadCell1;
			channels ++;
		}
		if (useLoadCell2 == true){
			average += i2cLoadCell2;
			channels ++;
		}
		if (useLoadCell3 == true){
			average += i2cLoadCell3;
			channels ++;
		}
		
		average = Math.round(average / channels);
		resolvedLoadCell = average;
		
		//if loadcell range is not zero, calc the weight
		if (loadCellSpan - loadCellZero){
			
			var newWeight = weightSpan * (resolvedLoadCell - loadCellZero);
			newWeight = newWeight / (loadCellSpan - loadCellZero);
			newWeight = parseInt(newWeight, 10);
			
			//apply the weight offset now
			newWeight += weightOffset;
			
			if (resetFilter){
				//reset data for moving ave filter
				sampleBuffer.fill(0);
				samples = 0;
				samplePointer = 0;
				sampleTotal = 0;
				resetFilter = false;
			}
			
			switch (filterMode){
				
				//filter off
				case 0:
					tempDisplayWeight = newWeight;
				break;
				
				
				//tracking filter, if display is too low, it will gradually increase
				//until feather time elapses, then apply coarse adjust
				case 1:
					featherTime = 15;
					deadBand = 5;
					fineAdjustStart = 4;
					fineAdjustUp += 2;
					fineAdjustDown += 2;
					coarseAdjust = 200;
					
					//too high
					if (displayWeight > (newWeight + deadBand))
					{
						tooHighCounter ++;
						tooLowCounter = 0;
						fineAdjustUp = 0;
						
						if (tooHighCounter < featherTime)
						{	
							tempDisplayWeight -= fineAdjustStart + fineAdjustDown;
						}
						else
						{
							tempDisplayWeight -= coarseAdjust;
							fineAdjustDown = 0;
							
							if (tempDisplayWeight < newWeight)
							{
								tempDisplayWeight = newWeight;
								tooHighCounter = 0;
							}
						}
					}
					//too low
					else if (displayWeight < (newWeight - deadBand))
					{
						tooLowCounter ++;
						tooHighCounter = 0;
						fineAdjustDown = 0;
						
						if (tooLowCounter < featherTime)
						{
							tempDisplayWeight += fineAdjustStart + fineAdjustUp;
						}
						else
						{
							tempDisplayWeight += coarseAdjust;
							fineAdjustUp = 0;
							
							if (tempDisplayWeight > newWeight)
							{
								tempDisplayWeight = newWeight;
								tooLowCounter = 0;
							}
						}
					}
					else
					{
						tooHighCounter = 0;
						tooLowCounter = 0;
						fineAdjustUp = 0;
						fineAdjustDown = 0;
					}
				break;
				
				
				//moving average filter
				case 2:
					numberOfSamples = 50;
					
					if (samples < numberOfSamples)
					{
						samples ++;
					}
					
					sampleTotal -= sampleBuffer[samplePointer];
					sampleTotal += newWeight;
					sampleBuffer[samplePointer] = newWeight;
					
					tempDisplayWeight = sampleTotal / samples;
					
					samplePointer ++;
					
					if (samplePointer == numberOfSamples)
					{
						samplePointer = 0;
					}
				break;
				
				default:
			}
			
			//if filter is on, snap to zero if within 50kg
			if (filterMode){
				if (tempDisplayWeight < 50 && tempDisplayWeight > -50){
					tempDisplayWeight = 0;
				}
			}
		}
		else {
			tempDisplayWeight = 0;
		}
		
		//update display, round to 1kg
		//displayWeight = Math.round(tempDisplayWeight);
		
		//if zero or span is the default value, load a big negative number
		//so the display prompts user to calibrate
		if (loadCellZero == 0 || loadCellSpan == 130000){
			displayWeight = -100000;
		}
		else {
			//update display, round to 10kg
			displayWeight = Math.round(tempDisplayWeight / 10) * 10;
		}
		
	}
	catch (e){
		console.log('Catch error processLoadCells');
	}
}

function writeLoadCellMcu(txData){
	try	{
		//message settings
		var slaveID = 80;
		var slaveRegister = 0;
		var packetLength = 15;
		
		//load end of frame byte
		txData[14] = 0xC3;
		
		//send block of bytes
		i2c1.writeI2cBlockSync(slaveID, slaveRegister, packetLength, txData);
		
		console.log('I2C loadcell mcu:          WRITE');
		
		//i2c1.closeSync();
	}
	catch (e){
		console.log('Catch error writeLoadCellMcu');
	}
}

function writeMainMcu(txData){
	try	{
		//message settings
		var slaveID = 60;
		var slaveRegister = 0;
		var packetLength = 15;
		
		//load end of frame byte
		txData[14] = 0xC3;
		
		//send block of bytes
		i2c1.writeI2cBlockSync(slaveID, slaveRegister, packetLength, txData);
		
		console.log('I2C main mcu:              WRITE');
		
		//i2c1.closeSync();
	}
	catch (e){
		console.log('Catch error writeMainMcu');
	}
}

function readMainMcu(){
	try	{
		//message settings
		var slaveID = 60;
		var slaveRegister = 100;
		var packetLength = 30;
		var packet = new Buffer(32, 0);
		
		//read block of bytes
		i2c1.readI2cBlockSync(slaveID, slaveRegister, packetLength, packet);
		
		//if valid end of frame byte
		//clr failure counter, and transfer the data
		if (packet[29] == 0xC3){
			mainMcuOnline = true;
			i2cMainMcuCounts ++;
				
			console.log('I2C mcu1 read data1:       OK   ' + i2cMainMcuCounts);
			
			i2cMainMcuFlags = join2bytes(packet[1], packet[2]);
			i2cSupplyVoltage = packet[3];
			i2cBeltSpeed = join2bytes(packet[4], packet[5]);
			i2cSpinnerSpeed = join2bytes(packet[6], packet[7]);
			i2cGroundSpeed = join2bytes(packet[8], packet[9]);
			i2cDoorSensorMv = join2bytes(packet[10], packet[11]);
			i2cTargetBeltSpeed = join2bytes(packet[12], packet[13]);
			i2cValve1Command = packet[14];
			i2cPwmInDuty = packet[15];
		}
		else {
			console.log('End of frame error readMainMcu');
			//reset this flag after 2secs, this resets the mcu i2c module
			mainMcuOnline = false;
			setTimeout(function(){ mainMcuOnline = true; }, 2000);
		}
	}
	catch (e){
		console.log('Catch error readMainMcu');
		//reset this flag after 2secs, this resets the mcu i2c module
		mainMcuOnline = false;
		setTimeout(function(){ mainMcuOnline = true; }, 2000);
	}
}


function processMainMcu(){
	try {
		
		/*
		//test values 100kg/ha urea, 21.8km/h, 25mm door, 720rpm spiner
		i2cBeltSpeed = 715;
		i2cSpinnerSpeed = 1200;
		i2cGroundSpeed = 1000;
		i2cDoorSensorMv = 9350;
		i2cTargetBeltSpeed = 715;
		
		if (masterOn){
			i2cBeltSpeed = 15000;
			i2cSpinnerSpeed = 1200;
			i2cGroundSpeed = 1000;
			i2cDoorSensorMv = 9350;
			i2cTargetBeltSpeed = 715;
		}
		else {
			i2cBeltSpeed = 0;
			i2cSpinnerSpeed = 0;
			i2cGroundSpeed = 0;
		}
		*/
		
		//main mcu sends a flag to identify when the master switch has been pressed
		//the flag just flip flops, it doesn't mean the master is on or off
		//the pi and ipad checks this flag, if it's changed, it knows it was pressed
		//get the initial value first, then after a few read cycles, allow master toggle
		var newI2cFlipFlop = 0;
		if (i2cMainMcuFlags & 0x10){
			newI2cFlipFlop = 1;
		}
		
		if (i2cMainMcuCounts > 3){
			
			if (i2cMasterFlipFlop != newI2cFlipFlop){
				//if it's changed, store the new value and toggle the master
				i2cMasterFlipFlop = newI2cFlipFlop;
				masterOn = !masterOn;
				
				//if it's turning on, set the door first
				if (masterOn == true){
					targetDoorHeightMm = storedDoorHeights[productPointer];
					setDoor();
				}
				//else master turned off, save the data storage file
				else {
					saveDataStorageFile();
				}
				
				setBelt();
			}
		}
		else {
			i2cMasterFlipFlop = newI2cFlipFlop;
		}
		
		//calc actual door height
		if (actuatorInstalled){
			actualDoorHeightMm = 245 - (i2cDoorSensorMv / 42.553);
		}
		else {
			actualDoorHeightMm = targetDoorHeightMm;
		}
		
		//calc actual rate to display
		calcActualRate();
		
		//calc hectares to add to ipad hectare counter
		calcHectareCounter();
		
		//spinner speed rpm rounded to nearest 10
		spinnerRpm = Math.round(i2cSpinnerSpeed * 0.06) * 10;
		
		//check if we need to fire any alerts
		processAlerts();
		
		//if something loaded the data storage counter, decrement it, when it hits zero, save the file
		if (saveDataTimer > 0){
			
			//console.log('save data timer: ' + saveDataTimer);
			
			saveDataTimer --;
			if (saveDataTimer == 0){
				saveDataStorageFile();
			}
		}
	}
	catch (e){
		console.log('Catch error processMainMcu');
	}
}


function calcActualRate(){
	try	{		
		//first calc kg dispensed for the cal sample, then calc the actual rate
		var actualDoorHeightM = actualDoorHeightMm / 1000;
		
		//calc the kg dispensed per second, div 100 because i2cBeltSpeed is x100
		var kgDispensed = (i2cBeltSpeed * doorWidth * actualDoorHeightM * rollerDiameter * 3.14159265359 * productCal) / (rollerPulsesPerRev * 100);
		
		//if we have dispensed kgs, the data has changed, so load a counter to save the data storage file
		if (kgDispensed > 0){
			saveDataTimer = 3;
		}
		
		//update the cal sample weight
		calSampleKg += kgDispensed;
		
		//if not using loadcells, process the kg counter
		if (!loadCellsEnabled){
			
			countdownKg -= kgDispensed;
			
			if (countdownKg < 0){
				countdownKg = 0;
				displayWeight = 0;
			}
			else {
				//update display, round to 10kg
				displayWeight = Math.round(countdownKg / 10) * 10;
			}
		}
		
		//if master is off, or zero target rate, or zero belt speed, show zero
		if (!masterOn || !targetRate || !i2cBeltSpeed){
			actualRate = 0;
			rawRate = 0;
			return;
		}
		
		//calc the ha covered and the rate, using sim or real groundspeed
		//sim groundspeed is pps x100, i2c groundspeed is pps x100
		if (simGroundSpeedPps > 0){
			var haCovered = (simGroundSpeedPps * distancePerPulse * spreadWidth) / 1000000;
			rawRate = Math.round(kgDispensed / haCovered);
		}
		else if (i2cGroundSpeed > 0){
			var haCovered = (i2cGroundSpeed * distancePerPulse * spreadWidth) / 1000000;
			rawRate = Math.round(kgDispensed / haCovered);
		}
		else {
			rawRate = 0;
		}
		
		
		
		//light method to smooth and snap to target rate
		if (rawRate && targetRate){
			
			var rawRatio = rawRate / targetRate;
			
			//if equal or above the target rate
			if (rawRatio >= 1){
				//if 0% to 10% error, just show the target rate
				if (rawRatio <= 1.10){
					rawRate = targetRate;
				}
				//if 10% to 20% error, take the band from 10% to 20% and stretch it
				//to fill the band from 0% to 20%, then add it to the target rate
				else if (rawRatio <= 1.20){
					var scaledError = (rawRatio - 1.10) * 2 * targetRate;
					rawRate = Math.round(targetRate + scaledError);
				}
				else {
					rawRate = Math.round(targetRate * rawRatio);
				}
			}
			//else below the target rate, process same as above
			else {
				if (rawRatio >= 0.90){
					rawRate = targetRate;
				}
				else if (rawRatio >= 0.80){
					var scaledError = (0.90 - rawRatio) * 2 * targetRate;
					rawRate = Math.round(targetRate - scaledError);
				}
				else {
					rawRate = Math.round(targetRate * rawRatio);
				}
			}
		}
		//update the actual rate
		actualRate = rawRate;
	}
	catch (e){
		console.log('Catch error calcActualRate');
	}
}


function calcHectareCounter(){
	try	{
		//calc hectares to add to ipad hectare counter
		if (masterOn && i2cGroundSpeed){
			instantHectares = (i2cGroundSpeed * distancePerPulse * spreadWidth) / 1000000;
		}
		else {
			instantHectares = 0;
		}
		
		//accumulate hectare counter
		hectareCounter += instantHectares;
		
		//if we have instant hectares, the data has changed, so load a counter to save the data storage file
		if (instantHectares > 0){
			saveDataTimer = 3;
		}
	}
	catch (e){
		console.log('Catch error calcHectareCounter');
	}
}


function setDoor(){
	try	{
		//clear the door kickout alert
		alert6 = false;
		
		//door is 15deg off vertical, this is the equivalent vertical opening
		//cap min/max value before sending to mcu
		if (targetDoorHeightMm < 10){
			targetDoorHeightMv = 10000;
		}
		else if (targetDoorHeightMm > 245){
			targetDoorHeightMv = 0;
		}
		else {
			var targetDoorHeightMv = (targetDoorHeightMm - 10) * 42.553;
			
			targetDoorHeightMv = Math.round(targetDoorHeightMv);
			targetDoorHeightMv = 10000 - targetDoorHeightMv;
		}
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x04;
		txData1[3] = highByte(targetDoorHeightMv);
		txData1[4] = lowByte(targetDoorHeightMv);
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	}
	catch (e){
		console.log('Catch error setDoor');
	}
}

function setBelt(){
	try {
		//the mainboard processor adjusts the belt pulses per second based on 
		//the wheel pulses per second, so we just send it a ratio value
		//it makes adjustments at 10Hz, see belt value spreadsheet for details
		
		//if master is on, calc the belt speed ratio, else set it to zero
		if (masterOn == true){
			var doorHeightInMetres;
			var pulsesPerTargetMass;
			
			doorHeightInMetres = targetDoorHeightMm / 1000;
			kgPerRevolution = doorWidth * doorHeightInMetres * rollerDiameter * 3.14159265359 * productCal;
			pulsesPerKg = rollerPulsesPerRev / kgPerRevolution;
			pulsesPerTargetMass = pulsesPerKg * targetRate;
			pulsesPerHa = 10000 / (spreadWidth * distancePerPulse);
			beltSpeedRatio = pulsesPerTargetMass / pulsesPerHa;
			
			//send belt speed ratio as x100, and mask it to 16bit
			beltSpeedRatio = beltSpeedRatio * 100;
			
			if (beltSpeedRatio > 0xffff){
				beltSpeedRatio = 0xffff;
			}
			else {
				beltSpeedRatio = beltSpeedRatio & 0xffff;
			}
		}
		else {
			beltSpeedRatio = 0;
		}
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x12;
		txData1[3] = highByte(beltSpeedRatio);
		txData1[4] = lowByte(beltSpeedRatio);
		txData1[5] = valve1Speed;
		txData1[6] = valve1Brake;
		txData1[7] = valve1TargetSize;
		txData1[8] = (valve1JumpStart * 2.55) & 0xff;
		txData1[9] = (manualValve1 * 2.55) & 0xff;
		txData1[10] = highByte(simGroundSpeedPps);
		txData1[11] = lowByte(simGroundSpeedPps);
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	}
	catch (e){
		console.log('Catch error setBelt');
	}
}

//text for each alert is preset in ipad app
function processAlerts(){
	try	{
		//alert1 spinner speed is low
		if (masterOn && i2cBeltSpeed){
			if (spinnerRpm < 200){
				alert1Counter ++;
			}
			else {
				alert1Counter = 0;
				alert1 = false;
			}
			
			//fire the alert
			if (alert1Counter > 5){
				alert1 = true;
			}
		}
		else {
			alert1Counter = 0;
			alert1 = false;
		}
		
		//alert2 belt is turning, when master switch is off or target is zero
		if (!masterOn || targetRate == 0){
			
			if (i2cBeltSpeed){
				alert2Counter ++;
				
				//fire the alert
				if (alert2Counter > 8){
					alert2 = true;
				}
			}
			else {
				alert2Counter = 0;
				alert2 = false;
			}
		}
		else {
			alert2Counter = 0;
			alert2 = false;
		}
		
		//alert3 output rate not matching the target rate
		if (masterOn && i2cGroundSpeed && targetRate){
			var rateRatio = actualRate / targetRate;
			
			if (rateRatio < 0.5 || rateRatio > 1.5){
				alert3Counter ++;
			}
			else {
				alert3Counter = 0;
				alert3 = false;
			}
			
			//fire the alert
			if (alert3Counter > 10){
				alert3 = true;
			}
		}
		else {
			alert3Counter = 0;
			alert3 = false;
		}
		
		//alert4 target door height is too high
		if (masterOn && targetDoorHeightMm > 245){
			alert4 = true;
		}
		else {
			alert4 = false;
		}
		
		//alert5 target door height is too low
		if (masterOn && targetDoorHeightMm < 15){
			alert5 = true;
		}
		else {
			alert5 = false;
		}
		
		//alert6 door height problem
		//door kickout flag is self clearing after 1sec
		//this alert remains true until a new door command is sent
		if (actuatorInstalled){
			if (i2cMainMcuFlags & 0x04){
				alert6 = true;
			}
		}
		else {
			alert6 = false;
		}
		
		//alert7 belt slip detected
		
	}
	catch (e){
		console.log('Catch error processAlerts');
	}
}



//concatena bytes and return hex value
function join2bytes(b1, b0){
	var x1 = padByte(b1) + padByte(b0);
	x1 = parseInt(x1, 16);
	return x1;
}

//concatena 3 bytes and return hex value
function join3Bytes(b2, b1, b0){
	var x1 = padByte(b2) + padByte(b1) + padByte(b0);
	x1 = parseInt(x1, 16);
	return x1;
}

//converts byte to string with padded zero
function padByte(b){
	if (b < 0x10){
		b = '0' + b.toString(16);
	}
	else {
		b = b.toString(16);
	}
	return b;
}

//extract low byte
function lowByte(bb){
	var b0 = bb & 0xFF;
	return b0;
}

//extract high byte
function highByte(bb){
	var b1 = (bb >> 8) & 0xFF;
	return b1;
}






//__________________________________________________________________________________________
//server for webpages, sockets, and http requests

var express = require('express');  
var app = express();
var server = require('http').createServer(app);
var io = require('socket.io')(server);

//when express receives a get request, send the index file
app.use(express.static(__dirname)); 
app.get('/', function(request, response, next) 
{	
	console.log('Received http get request');
	
    response.sendFile(__dirname + '/index.html');
});


//when express receives a post request, extract the data and respond
app.use(express.json());
app.use(express.urlencoded({ extended: true })); 
app.post('/', function(request, response, next) 
{	
	var bodyData = request.body;
	
	if (bodyData.command == 'refreshData'){
		
		//console.log('Http post request: ' + JSON.stringify(bodyData));
		
		//parse values from the body data
		var ipadTargetRate = parseFloat(bodyData.targetRate);
		
		//ipad master switch was pressed
		if (bodyData.masterPressed == '1'){
			//update master status, if turning on also set the door
			if (bodyData.implementOn == '1'){
				masterOn = true;
				targetDoorHeightMm = storedDoorHeights[productPointer];
				setDoor();
			}
			//else if turning off, set flag and save data storage file
			else {
				masterOn = false;
				saveDataStorageFile();
			}
			
			//update other values and send
			targetRate = ipadTargetRate;
			setBelt();
		}
		//else if rate has changed update values
		else if (targetRate != ipadTargetRate){
			targetRate = ipadTargetRate;
			setBelt();
		}
	}
	
	//send new data to to ipad
	response.send({
		implementOn: masterOn,
		actualRate: actualRate,
		displayWeight: displayWeight,
		spinnerRpm: spinnerRpm,
		hectareCounter: hectareCounter,
		alert1: alert1,
		alert2: alert2,
		alert3: alert3,
		alert4: alert4,
		alert5: alert5,
		alert6: alert6,
		alert7: alert7
	});
});


//socket on, notify when socket is connected
io.on('connection', function(socket){
	
	console.log('Socket is connected');
	
	//simple message from webpage
	socket.on('join', function(data){
		console.log(data);
		socket.emit('status', 'Online');	
	});
	
	//__________________________________________________________________________________________
	//quick set page
	
	//webpage requesting data from the pi
	socket.on('quickSetPage', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled,
			displayWeight: displayWeight,
			hectareCounter: hectareCounter,
			productName: productName,
			spreadWidth: spreadWidth,
			actuatorInstalled: actuatorInstalled
		});
	});
	
	//__________________________________________________________________________________________
	//products
	
	//webpage requesting data from the pi
	socket.on('products', function(data){
		socket.emit('newData',{
			productName: productName,
			spreadWidth: spreadWidth,
			targetDoorHeightMm: targetDoorHeightMm,
			actuatorInstalled: actuatorInstalled,
			productCal: productCal,
			calSampleKg: calSampleKg
		});
	});
	
	//inc the product pointer, reset the timeout to save the config file so
	//we store the active product
	socket.on('nextProduct', function (data){
		
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		productPointer ++;
		if (productPointer > 9){
			productPointer = 0;
		}
		
		productName = storedProducts[productPointer];
		productCal = storedCals[productPointer];
		spreadWidth = storedWidths[productPointer];
		targetDoorHeightMm = storedDoorHeights[productPointer];
	});
	
	//dec the product pointer, reset the timeout to save the config file so
	//we store the active product
	socket.on('prevProduct', function (data){
		
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		productPointer --;
		if (productPointer < 0){
			productPointer = 9;
		}
		
		productName = storedProducts[productPointer];
		productCal = storedCals[productPointer];
		spreadWidth = storedWidths[productPointer];
		targetDoorHeightMm = storedDoorHeights[productPointer];
	});
	
	//user updated product info, save it
	socket.on('productSaveName', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		productName = data;
		storedProducts[productPointer] = productName;
	});
	
	//user updated product info, save it
	socket.on('productSaveCal', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		productCal = data;
		storedCals[productPointer] = productCal;
	});
	
	//user updated product info, save it
	socket.on('productSaveWidth', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		spreadWidth = data;
		storedWidths[productPointer] = spreadWidth;
	});
	
	//user updated product info, save it
	socket.on('productSaveDoor', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		targetDoorHeightMm = data;
		storedDoorHeights[productPointer] = targetDoorHeightMm;
	});
	
	//start new cal sample
	socket.on('newCalSample', function (data){
		//zero the sample, refresh cal factor and door height
		calSampleKg = 0;
		saveDataStorageFile();
		productCal = storedCals[productPointer];
		targetDoorHeightMm = storedDoorHeights[productPointer];
		//set the door and belt to refresh pulses per kg etc
		setDoor();
		setBelt();
	});
	
	//user entered a cal sample weight
	socket.on('calSampleWeight', function (data){
		//if the entered kg and counted kg are valid
		if (data > 0 && calSampleKg > 0){
			clearTimeout(saveConfigTimer);
			saveConfigTimer = setTimeout(saveUserConfig, 4000);
			
			//the correction factor is the entered kg divided by the counted kg
			var calAdjust = data / calSampleKg;
			var adjustedCal = productCal * calAdjust;
			
			//use math round to set 3 decimal places, this method is fastest
			productCal = Math.round(adjustedCal * 1000) / 1000;
			storedCals[productPointer] = productCal;
		}
		//update to show the entered kg
		calSampleKg = data;
		saveDataStorageFile();
	});
	
	//__________________________________________________________________________________________
	//settings machine
	
	//webpage requesting data from the pi
	socket.on('setupMachine', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled,
			distancePerPulse: distancePerPulse,
			i2cGroundSpeed: i2cGroundSpeed,
			rollerPulsesPerRev: rollerPulsesPerRev,
			doorWidth: doorWidth,
			rollerDiameter: rollerDiameter,
			valve1JumpStart: valve1JumpStart,
			valve1Speed: valve1Speed,
			valve1Brake: valve1Brake,
			valve1TargetSize: valve1TargetSize,
			actuatorInstalled: actuatorInstalled
		});
	});
	
	//wheel sensor cal
	socket.on('wheelCal', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		distancePerPulse = data;
		//run set belt to update other output data
		setBelt();
	});
	
	//belt sensor cal
	socket.on('beltPulseCal', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		rollerPulsesPerRev = data;
		//run set belt to update other output data
		setBelt();
	});
	
	//set door width
	socket.on('doorWidthCal', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		doorWidth = data;
		//run set belt to update other output data
		setBelt();
	});
	
	//set roller dia
	socket.on('rollerDiaCal', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		rollerDiameter = data;
		//run set belt to update other output data
		setBelt();
	});
	
	//set valve jump start
	socket.on('valveJumpStart', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		valve1JumpStart = data;
		//run set belt to update other output data
		setBelt();
	});
	
	//set valve speed
	socket.on('valveSpeed', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		valve1Speed = data;
		//run set belt to update other output data
		setBelt();
	});
	
	//set valve damping
	socket.on('valveDamping', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		valve1Brake = data;
		//run set belt to update other output data
		setBelt();
	});
	
	//set valve green zone
	socket.on('valveZone', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		valve1TargetSize = data;
		//run set belt to update other output data
		setBelt();
	});
	
	//change actuator installed
	socket.on('changeActuator', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		actuatorInstalled = !actuatorInstalled;
	});
	
	
	//__________________________________________________________________________________________
	//settings device
	
	//webpage requesting data from the pi
	socket.on('setupDevice', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled
		});
	});
	
	//reset to factory settings
	socket.on('factoryReset', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		loadFactoryConfig();
		clearDataStorage();
	});
	
	//exit this app and start servermain
	function reStartApp(){
		var spawn = require('child_process').spawn;
		
		function startApp(){
			spawn('node', ['/home/pi/servermain/servermain'], {
				cwd: '/home/pi/servermain',	//current working directory for child process
				detached: true,				//detatched from the parent process
				stdio: "inherit"			//stay attached to the controlling terminal
			});
		}
		
		process.on('exit', startApp);
		process.exit();
	}
	
	//change app to variable rate
	socket.on('selectAppVR', function(data){
		//load the name of the app to start
		var appName = {
			appName: 'appVR'
		};
		
		//convert to json strings and write to the servermain config file
		var data = JSON.stringify(appName, null, 2);
		fs.writeFileSync('/home/pi/servermain/config.json', data);
		
		setTimeout(reStartApp, 2500);
	});
	
	//change app to fixed rate
	socket.on('selectAppFR', function(data){
		//load the name of the app to start
		var appName = {
			appName: 'appFR'
		};
		
		//convert to json strings and write to the servermain config file
		var data = JSON.stringify(appName, null, 2);
		fs.writeFileSync('/home/pi/servermain/config.json', data);
		
		setTimeout(reStartApp, 2500);
	});
	
	//change app to monitor
	socket.on('selectAppMON', function(data){
		//load the name of the app to start
		var appName = {
			appName: 'appMON'
		};
		
		//convert to json strings and write to the servermain config file
		var data = JSON.stringify(appName, null, 2);
		fs.writeFileSync('/home/pi/servermain/config.json', data);
		
		setTimeout(reStartApp, 2500);
	});
	
	//__________________________________________________________________________________________
	//settings factory
	
	//webpage requesting data from the pi
	socket.on('factoryPage', function(data){
		//nothing required
	});
	
	//save to factory settings
	socket.on('saveFactory', function (data){		
		saveFactoryConfig();
	});
	
	//__________________________________________________________________________________________
	//settings hopper without loadcells
	
	//webpage requesting data from the pi
	socket.on('setupKgCounter', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled,
			quickFillKgs: quickFillKgs
		});
	});
	
	//set hopper kgs
	socket.on('setHopperKg', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		countdownKg = data;
		quickFillKgs = data;
		displayWeight = Math.round(countdownKg / 10) * 10;
		saveDataStorageFile();
	});
	
	//quick fill hopper kgs to previous amount
	socket.on('quickFillHopperKg', function (data){
		countdownKg = quickFillKgs;
		displayWeight = Math.round(countdownKg / 10) * 10;
		saveDataStorageFile();
	});
	
	//enable loadcells
	socket.on('enableLoadCells', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		loadCellsEnabled = true;
		resetFilter = true;
	});
	
	//__________________________________________________________________________________________
	//settings hopper with loadcells
	
	//webpage requesting data from the pi
	socket.on('setupLoadCells', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode,
			displayWeight: displayWeight
		});
	});
	
	//quick zero, set a weight offset
	socket.on('cellQuickZero', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		weightOffset = weightOffset + (0 - displayWeight);
		displayWeight = 0;
	});
	
	//cal zero
	socket.on('cellCalZero', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		weightOffset = 0;
		displayWeight = 0;
		loadCellZero = resolvedLoadCell;
		resetFilter = true;
	});
	
	//cal span
	socket.on('cellCalSpan', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		weightSpan = data;
		loadCellSpan = resolvedLoadCell;
		displayWeight = data;
		resetFilter = true;
	});
	
	//set loadcells to factory cal
	socket.on('factoryLoadCells', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		factoryLoadCells();
	});
	
	//set filter mode
	socket.on('setFilter', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		filterMode ++;
		if (filterMode > 2)
		{
			filterMode = 0;
		}
		
		resetFilter = true;
	});
	
	//set channel on/off
	socket.on('cellChannel0', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell0 = !useLoadCell0;
	});
	
	//set channel on/off
	socket.on('cellChannel1', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell1 = !useLoadCell1;
	});
	
	//set channel on/off
	socket.on('cellChannel2', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell2 = !useLoadCell2;
	});
	
	//set channel on/off
	socket.on('cellChannel3', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		useLoadCell3 = !useLoadCell3;
	});
	
	//disable loadcells
	socket.on('disableLoadCells', function (data){
		clearTimeout(saveConfigTimer);
		saveConfigTimer = setTimeout(saveUserConfig, 4000);
		
		loadCellsEnabled = false;
	});
	
	
	//__________________________________________________________________________________________
	//settings hectare counter
	
	//webpage requesting data from the pi
	socket.on('setupHectares', function(data){
		socket.emit('newData',{
			loadCellsEnabled: loadCellsEnabled,
			hectareCounter: hectareCounter
		});
	});
	
	//clear hectare counter
	socket.on('zeroHectares', function (data){
		hectareCounter = 0;
		saveDataStorageFile();
	});
	
	//calibrate hectare counter
	socket.on('calHectares', function (data){
		var newHectares = data;
		if (hectareCounter > 0){
			distancePerPulse = distancePerPulse * (newHectares / hectareCounter);
			
			clearTimeout(saveConfigTimer);
			saveConfigTimer = setTimeout(saveUserConfig, 4000);
		}
		
		hectareCounter = data;
		saveDataStorageFile();
	});
	
	
	//__________________________________________________________________________________________
	//diag system general
	
	//webpage requesting data from the pi
	socket.on('diagGeneral', function(data){
		socket.emit('newData',{
			loadCellMcuOnline: loadCellMcuOnline,
			mainMcuOnline: mainMcuOnline,
			i2cLoadCellCounts: i2cLoadCellCounts,
			i2cMainMcuCounts: i2cMainMcuCounts,
			i2cSupplyVoltage: i2cSupplyVoltage,
			hardwareVer: hardwareVer,
			softwareVer: softwareVer
		});
	});
	
	
	//__________________________________________________________________________________________
	//diag loadcells
	
	//webpage requesting data from the pi
	socket.on('diagLoadCells', function(data){
		socket.emit('newData',{
			i2cLoadCellFlags: i2cLoadCellFlags,
			useLoadCell0: useLoadCell0,
			useLoadCell1: useLoadCell1,
			useLoadCell2: useLoadCell2,
			useLoadCell3: useLoadCell3,
			i2cLoadCell0: i2cLoadCell0,
			i2cLoadCell1: i2cLoadCell1,
			i2cLoadCell2: i2cLoadCell2,
			i2cLoadCell3: i2cLoadCell3,
			resolvedLoadCell: resolvedLoadCell,
			loadCellZero: loadCellZero,
			loadCellSpan: loadCellSpan,
			weightSpan: weightSpan,
			weightOffset: weightOffset,
			filterMode: filterMode,
			displayWeight: displayWeight
		});
	});
	
	//__________________________________________________________________________________________
	//diag belt
	
	//webpage requesting data from the pi
	socket.on('diagHydBelt', function(data){
		socket.emit('newData',{
			i2cMainMcuFlags: i2cMainMcuFlags,
			masterOn: masterOn,
			spreadWidth: spreadWidth,
			distancePerPulse: distancePerPulse,
			pulsesPerHa: pulsesPerHa,
			targetDoorHeightMm: targetDoorHeightMm,
			doorWidth: doorWidth,
			rollerDiameter: rollerDiameter,
			productCal: productCal,
			kgPerRevolution: kgPerRevolution,
			rollerPulsesPerRev: rollerPulsesPerRev,
			pulsesPerKg: pulsesPerKg,
			targetRate: targetRate,
			beltSpeedRatio: beltSpeedRatio,
			i2cGroundSpeed: i2cGroundSpeed,
			simGroundSpeedPps: simGroundSpeedPps,
			i2cTargetBeltSpeed: i2cTargetBeltSpeed,
			i2cBeltSpeed: i2cBeltSpeed,
			i2cValve1Command: i2cValve1Command,
			i2cPwmInDuty: i2cPwmInDuty,
			manualValve1: manualValve1,
			actualRate: actualRate,
			spinnerRpm: spinnerRpm,
			simGroundSpeedKph: simGroundSpeedKph
		});
	});
	
	//webpage master switch pressed
	socket.on('toggleMaster', function (data){
		masterOn = !masterOn;
		
		if (masterOn == true){
			targetDoorHeightMm = storedDoorHeights[productPointer];
			setDoor();
		}
		//else master turned off, save data storage file
		else {
			saveDataStorageFile();
		}
		
		setBelt();
	});
	
	socket.on('setRate', function (data){
		targetRate = data;
		setBelt();
	});
	
	socket.on('setManGroundSpd', function (data){
		//user enters simulated ground speed km/h, so calc the equivalent
		//sensor pulses per sec x100 using the distance per pulse value
		simGroundSpeedKph = data;
		simGroundSpeedPps = (simGroundSpeedKph * 100) / (distancePerPulse * 3.6);
		setBelt();
	});
	
	socket.on('setManValveCommand', function (data){
		manualValve1 = data;
		setBelt();
	});
	
	socket.on('stopHydTests', function (data){
		masterOn = false;
		simGroundSpeedKph = 0;
		simGroundSpeedPps = 0;
		manualValve1 = 0;
		setBelt();
	});
	
	//__________________________________________________________________________________________
	//diag door
	
	//webpage requesting data from the pi
	socket.on('diagDoor', function(data){
		socket.emit('newData',{
			i2cMainMcuFlags: i2cMainMcuFlags,
			i2cDoorSensorMv: i2cDoorSensorMv,
			targetDoorHeightMm: targetDoorHeightMm,
			actualDoorHeightMm: actualDoorHeightMm,
			actuatorInstalled: actuatorInstalled
		});
	});
	
	//set door to the target height
	socket.on('diagSetDoor', function (data){
		setDoor();
	});
	
	//manual move door
	socket.on('openDoor', function (data){
		//clear the door kickout alert
		alert6 = false;
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x08;
		txData1[3] = 0x00;
		txData1[4] = 0x00;
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	});
	
	//manual move door
	socket.on('closeDoor', function (data){
		//clear the door kickout alert
		alert6 = false;
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x09;
		txData1[3] = 0x00;
		txData1[4] = 0x00;
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	});
	
	//stop moving door
	socket.on('stopDoor', function (data){
		//clear the door kickout alert
		alert6 = false;
		
		//command bytes to send via i2c
		txData1[0] = 0xc2;
		txData1[1] = 0x00;
		txData1[2] = 0x10;
		txData1[3] = 0x00;
		txData1[4] = 0x00;
		txData1[5] = 0x00;
		txData1[6] = 0x00;
		txData1[7] = 0x00;
		txData1[8] = 0x00;
		txData1[9] = 0x00;
		txData1[10] = 0x00;
		txData1[11] = 0x00;
		txData1[12] = 0x00;
		txData1[13] = 0x00;
		txData1[14] = 0x00;
		
		//send command to mcu
		writeMainMcu(txData1);
	});
	
	
	//__________________________________________________________________________________________


});

//server port
server.listen(80);






